import doctest

from insights.parsers import ls_boot
from insights.parsers.ls_boot import LsBoot
from insights.tests import context_wrap

LS_BOOT = """
/boot:
total 187380
dr-xr-xr-x.  3 0 0     4096 Mar  4 16:19 .
dr-xr-xr-x. 19 0 0     4096 Jul 14 09:10 ..
-rw-r--r--.  1 0 0   123891 Aug 25  2015 config-3.10.0-229.14.1.el7.x86_64
drwxr-xr-x.  6 0 0      111 Jun 15 09:51 grub2

/boot/grub2:
total 36
drwxr-xr-x. 6 0 0  104 Mar  4 16:16 .
dr-xr-xr-x. 3 0 0 4096 Mar  4 16:19 ..
lrwxrwxrwx. 1 0 0   11 Aug  4  2014 menu.lst -> ./grub.cfg
-rw-r--r--. 1 0 0   64 Sep 18  2015 device.map
-rw-r--r--. 1 0 0 7040 Mar 29 13:30 grub.cfg
"""

LS_BOOT_LINKS = """
/boot:
total 187380
-rw-------.  1 root root 19143244 Mar  3 14:31 initramfs-2.6.32-504.el6.x86_64.img
lrwxrwxrwx.  1 root root       40 May 11 11:00 initrd -> initramfs-2.6.32-504.12.2.el6.x86_64.img
lrwxrwxrwx.  1 root root       34 May 11 11:00 vmlinuz -> vmlinuz-2.6.32-504.12.2.el6.x86_64
-rwxr-xr-x.  1 root root  4153904 Sep 16  2014 vmlinuz-2.6.32-504.el6.x86_64
"""


def test_ls_boot():
    ls_boot = LsBoot(context_wrap(LS_BOOT))
    assert '/boot' in ls_boot
    assert '/boot/grub2' in ls_boot
    assert ls_boot.dirs_of('/boot') == ['.', '..', 'grub2']

    grub2_files = ls_boot.files_of('/boot/grub2')
    assert 'menu.lst' in grub2_files
    assert 'device.map' in grub2_files
    assert 'grub.cfg' in grub2_files
    assert len(grub2_files) == 3

    boot_files = ls_boot.files_of('/boot')
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_files
    assert len(boot_files) == 1


def test_boot_links():
    ls_boot = LsBoot(context_wrap(LS_BOOT_LINKS))
    boot_files = ls_boot.files_of('/boot')
    assert '/boot' in ls_boot
    assert 'initramfs-2.6.32-504.el6.x86_64.img' in boot_files
    assert 'initrd' in boot_files
    assert 'vmlinuz' in boot_files
    assert 'vmlinuz-2.6.32-504.el6.x86_64' in boot_files


def test_doc_examples():
    env = {'bootdir': LsBoot(context_wrap(LS_BOOT))}
    failed, total = doctest.testmod(ls_boot, globs=env)
    assert failed == 0



"""
A method to get the spec name of some special symbolic_name in the uploader.json
"""

SPECIAL_SYMBOLIC_NAME_IN_UPLODER_JSON = {
    'getconf_pagesize': 'getconf_page_size',
    'krb5_conf_d': 'krb5',
    'limits_d': 'limits_conf',
    'machine_id1': 'machine_id',
    'machine_id2': 'machine_id',
    'machine_id3': 'machine_id',
    'modprobe_conf': 'modprobe',
    'modprobe_d': 'modprobe',
    'ps_auxwww': None,
    'redhat_access_proactive_log': None,
    'rh_mongodb26_conf': 'mongod_conf',
    'rpm__V_packages': 'rpm_V_packages',
}


def get_spec_name_by_symbolic_name(sname):
    # match a spec name to a symbolic name
    # some symbolic names need to be renamed to fit specs
    return SPECIAL_SYMBOLIC_NAME_IN_UPLODER_JSON.get(sname, sname)



import pytest
from insights.core import marshalling


def mar_unmar(o, use_value_list=False):
    marshalled = marshalling.marshal(o, use_value_list)
    unmarshalled = marshalling.unmarshal(marshalled)
    return marshalled, unmarshalled


def test_string_marshal():
    flag = "TEST_FLAG"
    _, unmarshalled = mar_unmar(flag)
    assert unmarshalled == {flag: True}


def test_dict_marshal():
    doc = {"foo": "bar"}
    _, unmarshalled = mar_unmar(doc)
    assert unmarshalled == doc


def test_bad_returns():
    with pytest.raises(TypeError):
        marshalling.marshal(True)
    with pytest.raises(TypeError):
        marshalling.marshal(1)
    with pytest.raises(TypeError):
        marshalling.marshal(1.0)
    with pytest.raises(TypeError):
        marshalling.marshal([])
    with pytest.raises(TypeError):
        marshalling.marshal(())
    with pytest.raises(TypeError):
        marshalling.marshal(set())


def test_none_marshal():
    ma, um = mar_unmar(None)
    assert um is None


def test_value_list():
    ma, um = mar_unmar("test", use_value_list=True)
    assert um == {"test": [True]}



"""
ClusterConf - file ``/etc/cluster/cluster.conf``
================================================

Stores a filtered set of lines from the cluster config file.  Because of the
filtering, the content as a whole will not parse as XML.  We use a
:class:`insights.core.LogFileOutput` parser class because, sadly, it's
easiest.
"""
from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.cluster_conf)
class ClusterConf(LogFileOutput):
    """
    Parse the ``/etc/cluster/cluster.conf`` file as a list of lines.  ``get``
    can be used to find lines containing one or more keywords.  Because of
    filters used on this file, we cannot parse this as XML.
    """
    pass



"""
Parsers for RabbitMQ
====================

Parsers included in this module are:

RabbitMQUsers - command ``/usr/sbin/rabbitmqctl list_users``
------------------------------------------------------------

RabbitMQQueues - command ``/usr/sbin/rabbitmqctl list_queues name messages consumers auto_delete``
--------------------------------------------------------------------------------------------------

RabbitMQEnv - file ``/etc/rabbitmq/rabbitmq-env.conf``
------------------------------------------------------
"""
from collections import namedtuple
from re import compile

from insights.core import CommandParser, SysconfigOptions
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.rabbitmq_users)
class RabbitMQUsers(CommandParser):

    def parse_content(self, content):
        self.data = {}
        for line in content[1:-1]:
            line_splits = line.split(None, 1)
            if len(line_splits) > 1:
                self.data[line_splits[0]] = line_splits[1][1:-1]


TRUE_FALSE = {'true': True, 'false': False}
"""dict: Dictionary for converting true/false strings to bool."""


@parser(Specs.rabbitmq_queues)
class RabbitMQQueues(CommandParser):
    """Parse the output of the `rabbitmqctl list_queues` command.

    The actual command is
    `rabbitmqctl list_queues name messages consumers auto_delete`.

    The four columns that are output are:

    1. name - The name of the queue with non-ASCII characters escaped as in C.
    2. messages - Sum of ready and unacknowledged messages (queue depth).
    3. consumers - Number of consumers.
    4. auto_delete - Whether the queue will be deleted automatically when no longer used.

    The output of the command looks like::

        cinder-scheduler        0       3       false
        cinder-scheduler.ha-controller  0       3       false
        cinder-scheduler_fanout_ea9c69fb630f41b2ae6120eba3cd43e0        8141    1   true
        cinder-scheduler_fanout_9aed9fbc3d4249289f2cb5ea04c062ab        8145    0   true
        cinder-scheduler_fanout_b7a2e488f3ed4e1587b959f9ac255b93        8141    0   true

    Examples:

        >>> queues.data[0]
        QueueInfo(name='cinder-scheduler', messages=0, consumers=3, auto_delete=False)
        >>> queues.data[0].name
        'cinder-scheduler'
        >>> queues.data[1].name
        'cinder-scheduler.ha-controller'

    Raises:
        ParseException: Raised if the data indicates an error in acquisition or if
            the auto_delete value is not true or false.
        ValueError: Raised if any of the numbers are not valid numbers
    """

    QueueInfo = namedtuple('QueueInfo', ['name', 'messages', 'consumers', 'auto_delete'])
    """namedtuple: Structure to hold a line of RabbitMQ queue information."""

    def parse_content(self, content):
        self.data = []
        for line in content:
            if "Listing queues ..." in line:
                continue
            if "...done." in line:
                continue
            parts = line.split()
            if len(parts) == 4 and not line.startswith('Error:'):
                if parts[3].lower() in TRUE_FALSE:
                    self.data.append(RabbitMQQueues.QueueInfo(
                        parts[0], int(parts[1]), int(parts[2]),
                        TRUE_FALSE[parts[3].lower()])
                    )
                else:
                    raise ParseException(
                        "auto_delete should be true or false: {0}".format(line))
            else:
                raise ParseException("Data appears invalid: {0}".format(line))


@parser(Specs.rabbitmq_env)
class RabbitMQEnv(SysconfigOptions):
    """Parse the content of file ``/etc/rabbitmq/rabbitmq-env.conf`` using
    the ``SysconfigOptions`` base class.


    Sample content of the file ``/etc/rabbitmq/rabbitmq-env.conf``::

        RABBITMQ_SERVER_ERL_ARGS="+K true +P 1048576 -kernel inet_default_connect_options [{nodelay,true},{raw,6,18,<<5000:64/native>>}] -kernel inet_default_listen_options [{raw,6,18,<<5000:64/native>>}]"

    Example:

        >>> rabbitmq_env.rabbitmq_server_erl_args
        '+K true +P 1048576 -kernel inet_default_connect_options [{nodelay,true},{raw,6,18,<<5000:64/native>>}] -kernel inet_default_listen_options [{raw,6,18,<<5000:64/native>>}]'
        >>> rabbitmq_env.data['RABBITMQ_SERVER_ERL_ARGS']
        '+K true +P 1048576 -kernel inet_default_connect_options [{nodelay,true},{raw,6,18,<<5000:64/native>>}] -kernel inet_default_listen_options [{raw,6,18,<<5000:64/native>>}]'
        >>> rabbitmq_env.rmq_erl_tcp_timeout
        '5000'

    Attributes:
        rabbitmq_server_erl_args(str): If RABBITMQ_SERVER_ERL_ARGS otherwise ``None``.

        rmq_erl_tcp_timeout(str): If value of inet_default_connect_options equals value of inet_default_listen_options. Otherwise ``None``.
    """
    @property
    def rabbitmq_server_erl_args(self):
        return self.data.get('RABBITMQ_SERVER_ERL_ARGS', None)

    @property
    def rmq_erl_tcp_timeout(self):
        pattern_connect_options = compile(r"inet_default_connect_options.*?\<<([0-9]*)\:64")
        pattern_listen_options = compile(r"inet_default_listen_options.*?\<<([0-9]*)\:64")

        if self.rabbitmq_server_erl_args:
            if pattern_connect_options.search(self.rabbitmq_server_erl_args) and pattern_listen_options.search(self.rabbitmq_server_erl_args):
                connect_timeout = pattern_connect_options.findall(self.rabbitmq_server_erl_args)[0]
                listen_timeout = pattern_listen_options.findall(self.rabbitmq_server_erl_args)[0]
                if connect_timeout == listen_timeout:
                    return connect_timeout



"""
BrctlShow - command ``brctl show``
==================================

This module provides processing for the output of the ``brctl show`` command.

Class ``BrctlShow`` parses the output of the ``brctl show`` command.
Sample output of this command looks like::

    ---
    bridge name     bridge id               STP enabled     interfaces
    br0             8000.08002731ddfd       no              eth1
                                                            eth2
                                                            eth3
    br1             8000.0800278cdb62       no              eth4
                                                            eth5
    br2             8000.0800278cdb63       no              eth6
    docker0         8000.0242d4cf2112       no
    ---

Examples:
    >>> brctl_content = '''
    ... bridge name     bridge id               STP enabled     interfaces
    ... br0             8000.08002731ddfd       no              eth1
    ...                                                         eth2
    ...                                                         eth3
    ... br1             8000.0800278cdb62       no              eth4
    ...                                                         eth5
    ... br2             8000.0800278cdb63       no              eth6
    ... docker0         8000.0242d4cf2112       no
    ... '''.strip()
    >>> from insights.parsers.brctl_show import BrctlShow
    >>> from insights.tests import context_wrap
    >>> shared = {BrctlShow: BrctlShow(context_wrap(brctl_content))}
    >>> brctl_info = BrctlShow(context_wrap(brctl_content))
    >>> brctl_info.data
    [
     {'interfaces': ['eth1', 'eth2', 'eth3'], 'bridge id': '8000.08002731ddfd',
      'STP enabled': 'no', 'bridge name': 'br0'},
     {'interfaces': ['eth4', 'eth5'], 'bridge id': '8000.0800278cdb62',
      'STP enabled': 'no', 'bridge name': 'br1'},
     {'interfaces': ['eth6'], 'bridge id': '8000.0800278cdb63',
      'STP enabled': 'no', 'bridge name': 'br2'},
     {'bridge id': '8000.0242d4cf2112', 'STP enabled': 'no',
      'bridge name': 'docker0'}
    ]
    >>> brctl_info.group_by_iface
    {
     'docker0': {'STP enabled': 'no', 'bridge id': '8000.0242d4cf2112'},
     'br2': {'interfaces': ['eth6'], 'STP enabled': 'no',
             'bridge id': '8000.0800278cdb63'},
     'br1': {'interfaces': ['eth4', 'eth5'], 'STP enabled': 'no',
             'bridge id': '8000.0800278cdb62'},
     'br0': {'interfaces': ['eth1', 'eth2', 'eth3'], 'STP enabled': 'no',
             'bridge id': '8000.08002731ddfd'}
    }
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.brctl_show)
class BrctlShow(CommandParser):
    """
    Parse the output of the command "brctl show" to get bridge
    interface info table
    """
    @property
    def group_by_iface(self):
        """
        Return a dict, key is the bridge name, the value is a dic with keys: bridge id,
        STP enabled and interfaces
        """
        return self._group_by_iface

    def parse_content(self, content):
        self._group_by_iface = {}
        self.data = []
        if "\t" in content[0]:
            head_line = list(filter(None, [v.strip() for v in content[0].split('\t')]))
        else:
            head_line = list(filter(None, [v.strip() for v in content[0].split('  ')]))
        iface = head_line[3] if len(head_line) >= 3 else None
        if iface:
            for line in content[1:]:
                if not line.startswith((' ', '\t')):
                    iface_lst = []
                    br_mapping = {}
                    br_mapping = dict(zip(head_line, line.split()))
                    if len(line.split()) == 4:
                        iface_lst.append(line.split()[3])
                        br_mapping[iface] = iface_lst
                    if br_mapping:
                        self.data.append(br_mapping)

                else:
                    iface_lst.append(line.strip())
                    br_mapping[iface] = iface_lst
            for entry in self.data:
                self._group_by_iface[entry['bridge name']] = \
                    dict((k, v) for (k, v) in entry.items() if k != 'bridge name')
        else:
            raise ParseException("Invalid Data Found")



import time

from insights.core import dr
from insights.core.context import HostContext, SosArchiveContext
from insights.core.exceptions import TimeoutException
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, RegistryPoint, SpecSet, foreach_execute


class Specs(SpecSet):
    spec_ds_timeout_1_2 = RegistryPoint()
    spec_ds_timeout_3_1 = RegistryPoint()
    spec_ds_timeout_default_1 = RegistryPoint()
    spec_foreach_ds_timeout_1_2 = RegistryPoint(multi_output=True)


@datasource(timeout=1)
def foreach_ds_timeout_1_2(broker):
    time.sleep(2)
    return ['test1', 'test2', 'test3']


@datasource(timeout=1)
def ds_timeout_1_2(broker):
    time.sleep(2)
    return DatasourceProvider('foo', "test_ds_timeout_1_2")


@datasource(timeout=3)
def ds_timeout_3_1(broker):
    time.sleep(1)
    return DatasourceProvider('foo', "test_ds_timeout_3_1_")


@datasource()
def ds_timeout_default_1(broker):
    time.sleep(1)
    return DatasourceProvider('foo', "test_ds_timeout_default_1")


class TestSpecs(Specs):
    spec_ds_timeout_1_2 = ds_timeout_1_2
    spec_ds_timeout_3_1 = ds_timeout_3_1
    spec_ds_timeout_default_1 = ds_timeout_default_1
    spec_foreach_ds_timeout_1_2 = foreach_execute(foreach_ds_timeout_1_2, "/usr/bin/echo %s")

#
# TEST
#


def test_timeout_datasource_no_hit():
    broker = dr.Broker()
    broker[HostContext] = HostContext()
    persist = set([
        TestSpecs.spec_ds_timeout_3_1,
        TestSpecs.spec_ds_timeout_default_1,
    ])

    dr.run_all(persist, broker=broker)

    assert ds_timeout_3_1 in broker
    assert ds_timeout_default_1 in broker


def test_timeout_datasource_hit():
    broker = dr.Broker()
    broker[HostContext] = HostContext()
    persist = set([
        TestSpecs.spec_ds_timeout_1_2,
        TestSpecs.spec_ds_timeout_default_1,
        TestSpecs.spec_ds_timeout_3_1,
    ])

    dr.run_all(persist, broker=broker)

    assert ds_timeout_3_1 in broker
    assert ds_timeout_default_1 in broker
    assert ds_timeout_1_2 not in broker
    assert Specs.spec_ds_timeout_1_2 in broker.exceptions
    exs = broker.exceptions[Specs.spec_ds_timeout_1_2]
    assert [ex for ex in exs if isinstance(ex, TimeoutException) and str(ex) == "Datasource spec insights.tests.datasources.test_datasource_timeout.TestSpecs.spec_ds_timeout_1_2 timed out after 1 seconds!"]


def test_timeout_foreach_datasource_hit():
    broker = dr.Broker()
    broker[HostContext] = HostContext()
    persist = set([
        TestSpecs.spec_ds_timeout_3_1,
        TestSpecs.spec_ds_timeout_default_1,
        TestSpecs.spec_foreach_ds_timeout_1_2,
    ])

    dr.run_all(persist, broker=broker)

    assert ds_timeout_3_1 in broker
    assert ds_timeout_default_1 in broker
    assert foreach_ds_timeout_1_2 not in broker
    assert Specs.spec_foreach_ds_timeout_1_2 in broker.exceptions
    exs = broker.exceptions[Specs.spec_foreach_ds_timeout_1_2]
    assert [ex for ex in exs if isinstance(ex, TimeoutException) and str(ex) == "Datasource spec insights.tests.datasources.test_datasource_timeout.foreach_ds_timeout_1_2 timed out after 1 seconds!"]


def test_not_hostcontext_timeout_datasource_hit():
    broker = dr.Broker()
    broker[SosArchiveContext] = SosArchiveContext()
    persist = set([
        TestSpecs.spec_ds_timeout_3_1,
        TestSpecs.spec_ds_timeout_default_1,
        TestSpecs.spec_ds_timeout_1_2,
    ])

    dr.run_all(persist, broker=broker)

    assert ds_timeout_3_1 in broker
    assert ds_timeout_default_1 in broker
    assert ds_timeout_1_2 in broker
    assert Specs.spec_ds_timeout_1_2 not in broker.exceptions


def test_not_hostcontext_timeout_foreach_datasource_hit():
    broker = dr.Broker()
    broker[SosArchiveContext] = SosArchiveContext()
    persist = set([
        TestSpecs.spec_ds_timeout_3_1,
        TestSpecs.spec_ds_timeout_default_1,
        TestSpecs.spec_foreach_ds_timeout_1_2
    ])

    dr.run_all(persist, broker=broker)

    assert ds_timeout_3_1 in broker
    assert ds_timeout_default_1 in broker
    assert foreach_ds_timeout_1_2 in broker
    assert Specs.spec_foreach_ds_timeout_1_2 not in broker.exceptions



import pytest
from insights.parsr import Char, Many


def test_many():
    a = Char("a")
    b = Char("b")
    x = Char("x")

    xs = Many(x)
    assert xs("") == []
    assert xs("a") == []
    assert xs("x") == ["x"]
    assert xs("xxxxx") == ["x", "x", "x", "x", "x"]
    assert xs("xxxxb") == ["x", "x", "x", "x"]

    ab = Many(a + b)
    assert ab("") == []
    assert ab("ba") == []
    assert ab("ab") == [["a", "b"]]
    assert ab("ababab") == [["a", "b"], ["a", "b"], ["a", "b"]]

    ab = Many(a | b)
    assert ab("aababb") == ["a", "a", "b", "a", "b", "b"]


def test_many1():
    a = Char("a")
    b = Char("b")
    x = Char("x")

    xs = Many(x, lower=1)
    with pytest.raises(Exception):
        xs("")

    with pytest.raises(Exception):
        xs("a")

    assert xs("x") == ["x"]
    assert xs("xxxxx") == ["x", "x", "x", "x", "x"]
    assert xs("xxxxb") == ["x", "x", "x", "x"]

    ab = Many(a + b, lower=1)
    with pytest.raises(Exception):
        ab("")

    with pytest.raises(Exception):
        ab("ba")

    assert ab("ab") == [["a", "b"]]
    assert ab("ababab") == [["a", "b"], ["a", "b"], ["a", "b"]]

    ab = Many(a | b, lower=1)
    assert ab("aababb") == ["a", "a", "b", "a", "b", "b"]



"""
Custom datasource for client metadata
"""
import json
import logging
import os
import yaml

from itertools import chain

from insights import package_info
from insights.client.constants import InsightsConstants as constants
from insights.core.blacklist import BLACKLISTED_SPECS
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent, ContentException
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider

try:
    from insights_client.constants import InsightsConstants as wrapper_constants
except ImportError:
    wrapper_constants = None


logger = logging.getLogger(__name__)


@datasource(HostContext)
def ansible_host(broker):
    """
    Custom datasource for ``ansible_host`` getting from insights-client
    configuration.

    Raises:
        SkipComponent: When there is no `ansible_host` is configured.

    Returns:
        str: The Ansible Hostname
    """
    insights_config = broker.get('client_config')
    if insights_config and insights_config.ansible_host:
        return DatasourceProvider(content=insights_config.ansible_host,
                                  relative_path='ansible_host')
    raise SkipComponent


@datasource(HostContext)
def basic_auth_insights_client(broker):
    """
    Custom datasource for ``username`` and ``password`` getting from insights-client
    configuration.

    Raises:
        SkipComponent: When there is no insights_config.

    Returns:
        dict: username/password exitsing boolean
    """
    insights_config = broker.get('client_config')
    result = {}
    if insights_config:
        if insights_config.username:
            result['username_set'] = True
        if insights_config.password:
            result['pass_set'] = True
        if result:
            return DatasourceProvider(content=json.dumps(result), relative_path='basic_conf')
    raise SkipComponent


@datasource(HostContext)
def blacklist_report(broker):
    """
    Custom datasource for ``blacklist_report`` getting from insights-client
    configuration.

    Returns:
        str: The JSON strings
    """
    def length(lst):
        '''
        Because of how the INI remove.conf is parsed,
        an empty value in the conf will produce
        the value [''] when parsed. Do not include
        these in the report
        '''
        return len(list(filter(None, lst)))

    redact_config = broker.get('redact_config')
    client_config = broker.get('client_config')

    ret = dict(
        obfuscate=False,
        obfuscate_hostname=False,
        commands=0,
        files=0,
        components=0,
        patterns=0,
        keywords=0,
        using_new_format=True,
        using_patterns_regex=False,
    )
    if client_config:
        ret.update(
            obfuscate=client_config.obfuscate,
            obfuscate_hostname=client_config.obfuscate_hostname,
        )
    if redact_config:
        ret.update(
            commands=length(redact_config.get('commands', [])),
            files=length(redact_config.get('files', [])),
            components=length(redact_config.get('components', [])),
            keywords=length(redact_config.get('keywords', [])),
            using_new_format=redact_config.get('new_format', True),
        )
        if isinstance(redact_config.get('patterns'), dict):
            ret.update(
                patterns=length(redact_config['patterns']['regex']),
                using_patterns_regex=True
            )
        else:
            ret.update(patterns=length(redact_config.get('patterns')))
    return DatasourceProvider(content=json.dumps(ret),
                              relative_path='blacklist_report')


@datasource(HostContext)
def blacklisted_specs(broker):
    """
    Custom datasource for ``blacklisted_specs`` getting from insights-client
    configuration.

    Raises:
        SkipComponent: When there is no `file-redaction` is configured.

    Returns:
        str: The JSON strings
    """
    if BLACKLISTED_SPECS:
        return DatasourceProvider(content=json.dumps({"specs": BLACKLISTED_SPECS}),
                                  relative_path='blacklisted_specs')
    raise SkipComponent


@datasource(HostContext)
def branch_info(broker):
    """
    Custom datasource for ``branch_info`` getting from insights-client
    configuration.

    Returns:
        str: The JSON strings
    """
    insights_config = broker.get('client_config')
    if insights_config:
        if insights_config.offline:
            branch_info = constants.default_branch_info
        else:
            branch_info = insights_config.branch_info
    return DatasourceProvider(content=json.dumps(branch_info),
                              relative_path='branch_info')


@datasource(HostContext)
def display_name(broker):
    """
    Custom datasource for ``display_name`` getting from insights-client
    configuration.

    Raises:
        SkipComponent: When there is no `display_name` is configured.

    Returns:
        str: The JSON strings
    """
    insights_config = broker.get('client_config')
    if insights_config and insights_config.display_name:
        return DatasourceProvider(content=insights_config.display_name,
                                  relative_path='display_name')
    raise SkipComponent


@datasource(HostContext)
def egg_release(broker):
    """
    Custom datasource for ``display_name`` getting from insights-client
    configuration.

    Raises:
        SkipComponent: When cannot get the `egg_release`.

    Returns:
        str: The JSON strings
    """
    egg_release = ''
    try:
        with open(constants.egg_release_file) as fil:
            egg_release = fil.read()
    except (IOError, MemoryError) as e:
        logger.debug('Could not read the egg release file: %s', str(e))

    if egg_release:
        return DatasourceProvider(content=egg_release,
                                  relative_path='egg_release')
    raise SkipComponent


@datasource(HostContext)
def tags(broker):
    """
    Custom datasource for ``tags`` getting from insights-client configuration.

    Raises:
        SkipComponent: When there is no `tags.yaml` is configured.
        ContentException: When any exceptions occur.

    Returns:
        str: The JSON strings
    """
    tags = None
    # Get the 'tags.yaml' from the host first.
    tags_file_path = constants.default_tags_file
    if os.path.isfile(tags_file_path):
        try:
            with open(tags_file_path) as fp:
                tags = yaml.safe_load(fp) or {}
        except (yaml.YAMLError, yaml.parser.ParserError) as e:
            # can't parse yaml from conf
            logger.error("Invalid YAML. Unable to load '%s'" % tags_file_path)
            raise ContentException('ERROR: Cannot parse %s.\n\nError details: \n%s\n' % (tags_file_path, e))

        # --START--
        # NOTE:
        # The following code is from the following function
        # - insights.client.data_collector.DataCollector._write_tags
        # Please keep them consistence before removing that.
        def f(k, v):
            if type(v) is list:
                col = []
                for val in v:
                    col.append(f(k, val))
                return list(chain.from_iterable(col))
            elif type(v) is dict:
                col = []
                for key, val in v.items():
                    col.append(f(k + ":" + key, val))
                return list(chain.from_iterable(col))
            else:
                return [{"key": k, "value": v, "namespace": constants.app_name}]
        t = []
        for k, v in tags.items():
            iv = f(k, v)
            t.append(iv)
        t = list(chain.from_iterable(t))
        # --END--

        if t:
            # The actual file path in archive:
            # - insights-archive-xxx/data/tags.json
            return DatasourceProvider(content=json.dumps(t),
                                      relative_path='tags.json')
        msg = "Empty YAML. Unable to load '%s'." % tags_file_path
        logger.error(msg)
        raise ContentException(msg)
    logger.debug("'%s' does not exist" % tags_file_path)
    raise SkipComponent("No such file '%s'." % tags_file_path)


@datasource(HostContext)
def version_info(broker):
    """
    Custom datasource for ``version_info`` getting from insights-client configuration.

    Returns:
        str: The JSON strings of version info
    """
    try:
        client_version = wrapper_constants.version
    except AttributeError:
        client_version = None

    version_info = {}
    version_info['core_version'] = '{0}-{1}'.format(package_info['VERSION'],
                                                    package_info['RELEASE'])
    version_info['client_version'] = client_version

    return DatasourceProvider(content=json.dumps(version_info),
                              relative_path='version_info')



"""
Leapp
=====
Parsers for parsing output of the
:py:mod:`insights.specs.datasources.leapp.leapp_report` datasource.

LeappMigrationResults - based on ``/etc/migration-results``
-----------------------------------------------------------

LeappReport - based on ``/var/log/leapp/leapp-report.json``
-----------------------------------------------------------
"""
import json

from insights.core import Parser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.leapp_migration_results)
class LeappMigrationResults(Parser, list):
    """
    Class for parsing the
    :py:mod:`insights.specs.datasources.leapp.migration_results` datasource.

    Examples:
        >>> type(leapp_migration_results)
        <class 'insights.parsers.leapp.LeappMigrationResults'>
        >>> len(leapp_migration_results)
        2
        >>> leapp_migration_results[0]['run_id'] == '1edff870-626d-41ba-854c-8f9dc8f20dc3'
        True
    """
    def parse_content(self, content):
        # content will never be empty if the parser got triggered
        self.extend(json.loads(''.join(content)))


@parser(Specs.leapp_report)
class LeappReport(Parser, list):
    """
    Class for parsing the
    :py:mod:`insights.specs.datasources.leapp.leapp_report` datasource.

    Examples:
        >>> type(leapp_report)
        <class 'insights.parsers.leapp.LeappReport'>
        >>> 'inhibitor' in leapp_report[0]['groups']
        True
        >>> "Use of NFS detected. " in leapp_report[0]['title']
        True
    """
    def parse_content(self, content):
        # content will never be empty if the parser got triggered
        self.extend(json.loads(''.join(content)))



"""
Custom datasource related PCP (Performance Co-Pilot)
"""
import datetime
import glob
import logging
import os

from insights.combiners.ps import Ps
from insights.combiners.services import Services
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent, ContentException
from insights.core.plugins import datasource
from insights.parsers.hostname import Hostname, HostnameDefault
from insights.parsers.ros_config import RosConfig

logger = logging.getLogger(__name__)

pcp_metrics = [
        'disk.dev.total',
        'hinv.ncpu',
        'kernel.all.cpu.idle',
        'kernel.all.pressure.cpu.some.avg',
        'kernel.all.pressure.io.full.avg',
        'kernel.all.pressure.io.some.avg',
        'kernel.all.pressure.memory.full.avg',
        'kernel.all.pressure.memory.some.avg',
        'mem.physmem',
        'mem.util.available',
]


@datasource(Services, HostContext)
def pcp_enabled(broker):
    """
    Returns:
        bool: True if pmproxy service is on in services

    Raises:
        SkipComponent: When pmproxy service is not enabled
    """
    if not broker[Services].is_on("pmproxy"):
        raise SkipComponent("pmproxy not enabled")
    return True


@datasource(HostContext)
def ros_collect(broker):
    """
    Returns:
        bool: True if "ros_collect=True" is set in "insights-client.conf"

    Raises:
        SkipComponent: When "ros_collect=True" is not set
    """
    insights_config = broker.get('client_config')
    if insights_config and insights_config.ros_collect is True:
        return True
    raise SkipComponent


@datasource(Ps, RosConfig, HostContext)
def pmlog_summary_args(broker):
    """
    Determines the pmlogger file and the metrics to collect via `pmlog_summary`
    spec.

    Returns:
        str: Full arguments string that will be passed to the `pmlogsummary`,
             which contains the `pmlogger` archive file and the required `metrics`.

    Raises:
        SkipComponent: Raises when meeting one of the following scenario:
                       - No pmlogger process is running
                       - No pmlogger file
                       - No "mandatory on" metrics in `config.ros`
    """
    pm_file = None
    try:
        ps = broker[Ps]
        if not ps.search(COMMAND_NAME__contains='pmlogger'):
            raise SkipComponent("No 'pmlogger' is running")

        pcp_log_date = (datetime.date.today() - datetime.timedelta(days=1)).strftime("%Y%m%d")
        pm_file = "/var/log/pcp/pmlogger/ros/{0}.index".format(pcp_log_date)

        if not (os.path.exists(pm_file) and os.path.isfile(pm_file)):
            raise SkipComponent("No pmlogger archive file: {0}".format(pm_file))

    except Exception as e:
        raise SkipComponent("Failed to check pmlogger file existence: {0}".format(str(e)))

    metrics = set()
    try:
        ros = broker[RosConfig]
        for spec in ros.specs:
            if spec.get('state') == 'mandatory on':
                metrics.update(spec.get('metrics').keys())

        if not metrics:
            raise SkipComponent("No 'mandatory on' metrics in config.ros")

    except Exception as e:
        raise SkipComponent("Failed to get pmlogger metrics: {0}".format(str(e)))

    return "{0} {1}".format(pm_file, ' '.join(sorted(metrics)))


@datasource(ros_collect, [Hostname, HostnameDefault], HostContext)
def pcp_raw_files(broker):
    """
    Return all existing paths of PCP RAW data files of yesterday.

    .. note::
        The "hostname" in the file paths will be removed by the "save_as"
        in the `pcp_raw_data` spec.  All the collected PCP RAW data files
        will be stored in "var/log/pcp/pmlogger" directory in the archive.

    Returns:
        list(str): list of file path

    Raises:
        ContentException: Raises when no such directory or number of PCP RAW
                          files less than 3.
    """
    hostnames = set()
    hnd = broker.get(HostnameDefault)
    hostnames.add(hnd.raw) if hnd else None
    hn = broker.get(Hostname)
    hostnames.update([hn.hostname, hn.fqdn]) if hn else None

    pm_cand = []
    yesterday = (datetime.date.today() - datetime.timedelta(days=1)).strftime("%Y%m%d")
    for hostname in sorted(hostnames):
        pm_root = os.path.join("/var/log/pcp/pmlogger/", hostname)
        if os.path.exists(pm_root):
            pm_files = glob.glob(os.path.join(pm_root, "{0}.*.xz".format(yesterday)))
            pm_files.append(os.path.join(pm_root, "{0}.index".format(yesterday)))
            pm_files = sorted(set(filter(lambda x: os.path.isfile(x), pm_files)))
            if len([pf for pf in pm_files if pf.endswith(('.index', '.meta.xz', '.0.xz'))]) >= 3:
                pm_cand.append((pm_files, os.path.getmtime(pm_files[-2])))  # timestamp of '.index'

    if not pm_cand:
        raise ContentException("No PCP data directory OR incomplete PCP RAW data")

    # return the "pm_files" which:
    # 1. has latest yesterday ".index"
    # 2. has complete data (>=3)
    # 3. has much more data than others
    #    - a rare case: when multiple pm_files satisfy with both 1 and 2
    return sorted(pm_cand, key=lambda x: (x[1], len(x[0])))[-1][0]


@datasource(pcp_raw_files, HostContext)
def pmlog_summary_args_pcp_zeroconf(broker):
    """
    Return the 'index' of the PCP Raw data files and the required metrics.

    Returns:
        str: Full arguments string that will be passed to the `pmlogsummary`,
             which contains the `pmlogger` index file and the required `metrics`.
    """
    pm_files = broker.get(pcp_raw_files)
    pm_index = [fil for fil in pm_files if fil.endswith('.index')][0]
    return "{0} {1}".format(pm_index, ' '.join(pcp_metrics))



"""
SmartpdcSettings - file ``/etc/smart_proxy_dynflow_core/settings.yml``
======================================================================

This module provides parsing for smart_proxy_dynflow_core settings file.
``SmartpdcSettings`` is a parser for ``/etc/smart_proxy_dynflow_core/settings.yml`` files.

Typical output is::

    # Path to dynflow database, leave blank for in-memory non-persistent database
    :database:
    :console_auth: true

    # URL of the foreman, used for reporting back
    :foreman_url: https://test.example.com

    # SSL settings for client authentication against foreman.
    :foreman_ssl_ca: /etc/foreman-proxy/foreman_ssl_ca.pem
    :foreman_ssl_cert: /etc/foreman-proxy/foreman_ssl_cert.pem
    :foreman_ssl_key: /etc/foreman-proxy/foreman_ssl_key.pem

    # Listen on address
    :listen: 0.0.0.0

    # Listen on port
    :port: 8008

Examples:
    >>> smartpdc_settings.data[':foreman_url']
    'https://test.example.com'
    >>> "/etc/foreman-proxy/foreman_ssl_ca.pem" in smartpdc_settings.data[':foreman_ssl_ca']
    True
"""
from insights.specs import Specs

from .. import YAMLParser, parser


@parser(Specs.smartpdc_settings)
class SmartpdcSettings(YAMLParser):
    """ Class for parsing the content of ``/etc/smart_proxy_dynflow_core/settings.yml``."""
    pass



import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import multipath_conf
from insights.parsr.query import first, last
from insights.tests import context_wrap


CONF = """
blacklist {
       device {
               vendor  "IBM"
               product "3S42"       #DS4200 Product 10
       }
       device {
               vendor  "HP"
               product "*"
       }
}""".strip()


MULTIPATH_CONF_INFO = """
defaults {
       udev_dir                /dev
       path_selector           "round-robin 0"
       user_friendly_names     yes
}

multipaths {
       multipath {
               alias                   yellow
               path_grouping_policy    multibus
       }
       multipath {
               wwid                    1DEC_____321816758474
               alias                   red
       }
}

devices {
       device {
               path_selector           "round-robin 0"
               no_path_retry           queue
       }
       device {
               vendor                  "COMPAQ  "
               path_grouping_policy    multibus
       }
}

blacklist {
       wwid 26353900f02796769
       devnode "^hd[a-z]"
}

""".strip()


MULTIPATH_CONF_CASE_1 = """
defaults {
    enable_foreign "" # line added by Leapp
    allow_usb_devices yes # line added by Leapp
    find_multipaths yes
    user_friendly_names yes
}


blacklist {
}
"""

INPUT_EMPTY = ''

MULTIPATH_CONF_INITRAMFS_CMD_ERROR_1 = """
/usr/bin/timeout: failed to run command `/bin/lsinitrd': No such file or directory
"""
MULTIPATH_CONF_INITRAMFS_CMD_ERROR_2 = """
timeout: failed to run command `/bin/lsinitrd': No such file or directory
"""
MULTIPATH_CONF_INITRAMFS_CMD_ERROR_3 = """
No <initramfs file> specified and the default image '/boot/initramfs-4.18.0-193.el8.x86_64.img' cannot be accessed!

Usage: lsinitrd [options] [<initramfs file> [<filename> [<filename> [...] ]]]
Usage: lsinitrd [options] -k <kernel version>

-h, --help                  print a help message and exit.
-s, --size                  sort the contents of the initramfs by size.
-m, --mod                   list modules.
-f, --file <filename>       print the contents of <filename>.
--unpack                    unpack the initramfs, instead of displaying the contents.
                            If optional filenames are given, will only unpack specified files,
                            else the whole image will be unpacked. Won't unpack anything from early cpio part.
--unpackearly               unpack the early microcode part of the initramfs.
                            Same as --unpack, but only unpack files from early cpio part.
-v, --verbose               unpack verbosely.
-k, --kver <kernel version> inspect the initramfs of <kernel version>.
"""


def test_multipath_conf_tree():
    multipath_conf_info = multipath_conf.MultipathConfTree(context_wrap(MULTIPATH_CONF_INFO))
    assert 'defaults' in multipath_conf_info
    assert 'multipaths' in multipath_conf_info
    assert 'someunexistkey' not in multipath_conf_info
    assert multipath_conf_info['defaults']['udev_dir'].value == '/dev'
    assert multipath_conf_info['defaults']['path_selector'].value == 'round-robin 0'
    assert multipath_conf_info['multipaths']['multipath'][1]['alias'].value == 'red'
    assert multipath_conf_info['devices']['device'][0]['no_path_retry'].value == 'queue'
    assert multipath_conf_info['blacklist']['devnode'].value == '^hd[a-z]'

    assert 'path_selector' in multipath_conf_info['defaults']
    assert 'unexist_in_defaults' not in multipath_conf_info['defaults']
    assert bool(multipath_conf_info['defaults']['unexist_key']) is False
    assert multipath_conf_info['defaults']['unexist_key'].value is None

    assert [mp['alias'].value for mp in multipath_conf_info['multipaths']['multipath']] == ['yellow', 'red']
    assert [mp['wwid'].value for mp in multipath_conf_info['multipaths']['multipath']] == [None, '1DEC_____321816758474']


def test_multipath_conf_tree_initramfs():
    multipath_conf_info = multipath_conf.MultipathConfTreeInitramfs(context_wrap(MULTIPATH_CONF_INFO))
    assert multipath_conf_info['defaults']['udev_dir'].value == '/dev'
    assert multipath_conf_info['defaults']['path_selector'].value == 'round-robin 0'
    assert multipath_conf_info['multipaths']['multipath'][1]['alias'].value == 'red'
    assert multipath_conf_info['devices']['device'][0]['no_path_retry'].value == 'queue'
    assert multipath_conf_info['blacklist']['devnode'].value == '^hd[a-z]'


def test_multipath_conf_tree_special_handling():
    multipath_conf_info = multipath_conf.MultipathConfTree(context_wrap(MULTIPATH_CONF_CASE_1))
    assert multipath_conf_info['defaults']['enable_foreign'].value == '""'
    assert multipath_conf_info['defaults']['allow_usb_devices'].value == 'yes'


def test_multipath_conf_trees():
    for c in (multipath_conf.MultipathConfTree,
              multipath_conf.MultipathConfTreeInitramfs):
        conf = c(context_wrap(CONF))
        assert len(conf["blacklist"]) == 1
        assert len(conf["blacklist"]["device"]) == 2

        assert len(conf["blacklist"]["device"]["vendor"]) == 2
        assert len(conf["blacklist"]["device"]["product"]) == 2

        assert conf["blacklist"]["device"]["vendor"][first].value == "IBM"
        assert conf["blacklist"]["device"]["vendor"][last].value == "HP"

        assert conf["blacklist"]["device"]["product"][first].value == "3S42"
        assert conf["blacklist"]["device"]["product"][last].value == "*"


def test_empty_multipath_conf_tree():
    with pytest.raises(SkipComponent) as e_info:
        multipath_conf.MultipathConfTree(context_wrap(INPUT_EMPTY))
    assert "Empty content." in str(e_info.value)

    with pytest.raises(SkipComponent) as e_info:
        multipath_conf.MultipathConfTreeInitramfs(context_wrap(INPUT_EMPTY))
    assert "Empty content." in str(e_info.value)


def test_multipath_conf_tree_initramfs_errors():
    with pytest.raises(SkipComponent) as e_info:
        multipath_conf.MultipathConfTreeInitramfs(context_wrap(MULTIPATH_CONF_INITRAMFS_CMD_ERROR_1))
    assert "`/bin/lsinitrd': No such file or directory" in str(e_info.value)
    with pytest.raises(SkipComponent) as e_info:
        multipath_conf.MultipathConfTreeInitramfs(context_wrap(MULTIPATH_CONF_INITRAMFS_CMD_ERROR_2))
    assert "`/bin/lsinitrd': No such file or directory" in str(e_info.value)
    with pytest.raises(SkipComponent) as e_info:
        multipath_conf.MultipathConfTreeInitramfs(context_wrap(MULTIPATH_CONF_INITRAMFS_CMD_ERROR_3))
    assert "No <initramfs file> specified and the default image" in str(e_info.value)



import pytest
from mock.mock import patch, mock_open
from insights.core.dr import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs.datasources.sys_fs_cgroup_memory import sys_fs_cgroup_uniq_memory_swappiness

RELATIVE_PATH = 'insights_commands/sys_fs_cgroup_uniq_memory_swappiness'
EXPECTED_RESULT_1 = "60  3"
EXPECTED_RESULT_2 = """
60   1
10   2
""".strip()


@patch('insights.specs.datasources.sys_fs_cgroup_memory.open', new_callable=mock_open, read_data='60')
@patch('insights.specs.datasources.sys_fs_cgroup_memory.os.walk')
def test_sys_fs_cgroup_uniq_memory_swappiness_single_line(mock_walk, mock_ds_open):
    mock_walk.return_value = [
        ("/sys/fs/cgroup/memory", (), ("memory.swappiness", "test",)),
        ("/sys/fs/cgroup/memory/test_1", (), ("memory.swappiness", "test",)),
        ("/sys/fs/cgroup/memory/test_2", (), ("memory.swappiness", "test",)),
    ]

    broker = {}
    result = sys_fs_cgroup_uniq_memory_swappiness(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected = DatasourceProvider(content=EXPECTED_RESULT_1, relative_path=RELATIVE_PATH)
    assert len(result.content) == len(expected.content) == 1
    assert result.content[0].split() == expected.content[0].split()
    assert result.relative_path == expected.relative_path


@patch('insights.specs.datasources.sys_fs_cgroup_memory.open', new_callable=mock_open, read_data='60')
@patch('insights.specs.datasources.sys_fs_cgroup_memory.os.walk')
def test_sys_fs_cgroup_uniq_memory_swappiness_multi_lines(mock_walk, mock_ds_open):
    mock_walk.return_value = [
        ("/sys/fs/cgroup/memory", (), ("memory.swappiness", "test",)),
        ("/sys/fs/cgroup/memory/test_1", (), ("memory.swappiness", "test",)),
        ("/sys/fs/cgroup/memory/test_2", (), ("memory.swappiness", "test",)),
    ]

    mock_ds_open.side_effect = (mock_ds_open.return_value, mock_open(read_data="10").return_value, mock_open(read_data="10").return_value,)

    broker = {}
    result = sys_fs_cgroup_uniq_memory_swappiness(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected = DatasourceProvider(content=EXPECTED_RESULT_2, relative_path=RELATIVE_PATH)
    assert len(result.content) == len(expected.content) == 2
    assert result.content[1].split() == expected.content[1].split()
    assert result.relative_path == expected.relative_path


@patch('insights.specs.datasources.sys_fs_cgroup_memory.open', new_callable=mock_open, read_data='60')
@patch('insights.specs.datasources.sys_fs_cgroup_memory.os.walk')
def test_sys_fs_cgroup_uniq_memory_swappiness_without_swappiness_file(mock_walk, mock_ds_open):
    mock_walk.return_value = [
        ("/sys/fs/cgroup/memory", (), ("cpu", "test",)),
        ("/sys/fs/cgroup/memory/test_1", (), ("cpu", "test",)),
        ("/sys/fs/cgroup/memory/test_2", (), ("cpu", "test",)),
    ]

    broker = {}
    with pytest.raises(SkipComponent) as e:
        sys_fs_cgroup_uniq_memory_swappiness(broker)
    assert 'SkipComponent' in str(e)


@patch('insights.specs.datasources.sys_fs_cgroup_memory.open', new_callable=mock_open, read_data='60')
@patch('insights.specs.datasources.sys_fs_cgroup_memory.os.walk')
def test_sys_fs_cgroup_uniq_memory_swappiness_without_memory_dir(mock_walk, mock_ds_open):
    mock_walk.return_value = []

    broker = {}
    with pytest.raises(SkipComponent) as e:
        sys_fs_cgroup_uniq_memory_swappiness(broker)
    assert 'SkipComponent' in str(e)



"""
FSTab - file ``/etc/fstab``
===========================

Parse the ``/etc/fstab`` file into a list of lines.  Each line is a dictionary
of fields, named according to their definitions in ``man fstab``:

* ``fs_spec`` - the device to mount
* ``fs_file`` - the mount point
* ``fs_vfstype`` - the type of file system
* ``fs_mntops`` - the mount options as a dictionary
* ``fs_freq`` - the dump frequency
* ``fs_passno`` - check the filesystem on reboot in this pass number
* ``raw_fs_mntops`` - the mount options as a string
* ``raw`` - the RAW line which is useful to front-end

``fs_freq`` and ``fs_passno`` are recorded as integers if found, and zero if
not present.

``fs_mntops`` is wrapped as a as a :class:`insights.parsers.mount.MountOpts`
object.  For instance, the option ``rw`` in ``rw,dmode=0500`` may be accessed as
``mnt_row_info.rw`` with the value ``True``, and the ``dmode`` can be accessed
as ``mnt_row_info.dmode`` with the value ``0500``.

This data, as above, is available in the ``data`` property:

* Wrapped as an :class:`FSTabEntry`, each column can also be accessed as an
  attribute with the same name.

The :class:`FSTabEntry` for each mount point is also available via the
:attr:`FSTab.mounted_on` property; the data is the same as that stored in the
:attr:`FSTab.data` list.

"""


from insights import Parser, parser, get_active_lines
from insights.specs import Specs
from insights.parsers import optlist_to_dict, parse_delimited_table, keyword_search
from insights.parsers.mount import MountOpts, AttributeAsDict

FS_HEADINGS = "fs_spec fs_file fs_vfstype raw_fs_mntops fs_freq fs_passno"


class FSTabEntry(AttributeAsDict):
    """
    An object representing an entry in ``/etc/fstab``.  Each entry contains
    below fixed attributes:

    Attributes:
        fs_spec (str): the device to mount
        fs_file (str): the mount point
        fs_vfstype (str): the type of file system
        fs_mntops (dict): the mount options as a :class:`insights.parser.mount.MountOpts`
        fs_freq (int): the dump frequency
        fs_passno (int): check the filesystem on reboot in this pass number
        raw_fs_mntops (str): the mount options as a string
        raw (str): the RAW line which is useful to front-end
    """
    pass


@parser(Specs.fstab)
class FSTab(Parser):
    """
    Parse the content of ``/etc/fstab``.

    Typical content of the ``fstab`` looks like::

        #
        # /etc/fstab
        # Created by anaconda on Fri May  6 19:51:54 2016
        #
        /dev/mapper/rhel_hadoop--test--1-root /                       xfs     defaults        0 0
        UUID=2c839365-37c7-4bd5-ac47-040fba761735 /boot               xfs     defaults        0 0
        /dev/mapper/rhel_hadoop--test--1-home /home                   xfs     defaults        0 0
        /dev/mapper/rhel_hadoop--test--1-swap swap                    swap    defaults        0 0

        /dev/sdb1 /hdfs/data1 xfs rw,relatime,seclabel,attr2,inode64,noquota 0 0
        /dev/sdc1 /hdfs/data2 xfs rw,relatime,seclabel,attr2,inode64,noquota 0 0
        /dev/sdd1 /hdfs/data3 xfs rw,relatime,seclabel,attr2,inode64,noquota 0 0

        localhost:/ /mnt/hdfs nfs rw,vers=3,proto=tcp,nolock,timeo=600 0 0

        /dev/mapper/vg0-lv2     /test1     ext4 defaults,data=writeback     1 1
        nfs_hostname.redhat.com:/nfs_share/data     /srv/rdu/cases/000  nfs     ro,defaults,hard,intr,bg,noatime,nodev,nosuid,nfsvers=3,tcp,rsize=32768,wsize=32768     0

    Examples:

        >>> type(fstab)
        <class 'insights.parsers.fstab.FSTab'>
        >>> len(fstab)
        9
        >>> fstab.data[0]['fs_spec'] # Note that data is a list not a dict here
        '/dev/mapper/rhel_hadoop--test--1-root'
        >>> fstab.data[0].fs_spec
        '/dev/mapper/rhel_hadoop--test--1-root'
        >>> fstab.data[0].raw
        '/dev/mapper/rhel_hadoop--test--1-root /                       xfs    defaults        0 0'
        >>> fstab.data[0].fs_mntops.defaults
        True
        >>> 'relatime' in fstab.data[0].fs_mntops
        False
        >>> fstab.data[0].fs_mntops.get('relatime')
        None
        >>> fstab.mounted_on['/hdfs/data3'].fs_spec
        '/dev/sdd1'


    Attributes:
        data (list): a list of parsed fstab entries as :class:`FSTabEntry` objects.
        mounted_on (dict): a dictionary of :class:`FSTabEntry` objects keyed on mount point.
    """

    def __len__(self):
        return len(self.data)

    def __iter__(self):
        for row in self.data:
            yield row

    def parse_content(self, content):
        """
        Parse each line in the file ``/etc/fstab``.
        """
        fstab_output = parse_delimited_table([FS_HEADINGS] + get_active_lines(content), raw_line_key='raw')
        self.data = []
        for line in fstab_output:
            line['fs_spec'] = line.get('fs_spec', '')
            line['fs_vfstype'] = line.get('fs_vfstype', '')
            # Decode fs_file to transfer the '\040' to ' '.
            # Encode first and then decode works for both Python2 and Python3.
            line['fs_file'] = line.get('fs_file', '').encode().decode("unicode-escape")
            line['fs_freq'] = int(line.get('fs_freq', '0'))
            line['fs_passno'] = int(line.get('fs_passno', '0'))
            # if there is no mntops, it is defaults.
            # (/dev/foo /foo somefs defaults   0 0) and (/dev/foo /foo somefs) are same
            line['raw_fs_mntops'] = line.get('raw_fs_mntops', 'defaults')
            # optlist_to_dict converts 'key=value' to key: value and 'key' to key: True
            line['fs_mntops'] = MountOpts(optlist_to_dict(line.get('raw_fs_mntops')))
            self.data.append(FSTabEntry(line))
        # assert: all mount points of valid entries are unique by definition
        self.mounted_on = dict((row.fs_file, row) for row in self.data)

    def search(self, **kwargs):
        """
        Search for the given key/value pairs in the data.  Please refer to the
        :py:meth:`insights.parsers.keyword_search` function documentation for
        a more complete description of how to use this.

        Fields that can be searched (as per ``man fstab``):

        * ``fs_spec``: the block special or remote filesystem path or label.
        * ``fs_file``: The mount point for the filesystem.
        * ``fs_vfstype``: The file system type.
        * ``fs_mntops``: The mount options.  Since this is also a dictionary,
          this can be searched using __contains - see the examples below.
        * ``fs_freq``: The dump frequency - rarely used.
        * ``fs_passno``: The pass for file system checks - rarely used.

        Examples:

            Search for the root file system:
                ``fstab.search(fs_file='/')``
            Search for file systems mounted from a LABEL declaration
                ``fstab.search(fs_spec__startswith='LABEL=')``
            Search for file systems that use the 'uid' mount option:
                ``fstab.search(fs_mntops__contains='uid')``
            Search for XFS file systems using the 'relatime' option:
                ``fstab.search(fs_vfstype='xfs', fs_mntops__contains='relatime')``
        """
        return keyword_search(self.data, **kwargs)

    def fsspec_of_path(self, path):
        """
        Return the device name if longest-matched mount-point of path is found,
        else None.
        If path contains any blank, pass it in directly or escape with '\040',
        eg: '/VM TOOLS/cache' or '/VM\040TOOLS/cache'
        """
        path = path.strip() if path else path
        if not path or not path.startswith('/'):
            return

        mos = self.mounted_on
        mps = sorted(mos, reverse=True)

        path = path if path.endswith('/') else path + '/'
        for mp in mps:
            tmp = mp if mp.endswith('/') else mp + '/'
            if path.startswith(tmp):
                return mos[mp].get('fs_spec', None)



import pytest
import re

from insights.core.exceptions import ParseException
from insights.parsers import ethtool
from insights.tests import context_wrap
from insights.util import keys_in

import doctest

TEST_ETHTOOL_A_DOCS = '''
Pause parameters for eth0:
Autonegotiate:  on
RX:             on
TX:             on
RX negotiated:  off
TX negotiated:  off
'''

SUCCESS_ETHTOOL_A = """
Pause parameters for enp0s25:
Autonegotiate : on
RX:             off
TX:             off
RX negotiated:  off
TX negotiated:  off
""".strip()

SUCCESS_ETHTOOL_A_PATH = """
sos_commands/networking/ethtool_-a_enp0s25
""".strip()

FAIL_ETHTOOL_A = """
Cannot get device pause settings: Operation not supported
Pause parameters for __wlp3s0:
""".strip()

FAIL_ETHTOOL_A_PATH = """
sos_commands/networking/ethtool_-a___wlp3s0
""".strip()

FAIL_ETHTOOL_A_1 = """
ethtool: bad command line argument(s)
For more information run ethtool -h
""".strip()
FAIL_ETHTOOL_A_PATH_1 = """
sos_commands/networking/ethtool_-a_bond0.1384@bond0
""".strip()

FAIL_ETHTOOL_A_2 = """
ethtool version 6
Usage:
ethtool DEVNAME Display standard information about device
    ethtool -s|--change DEVNAME
""".strip()
FAIL_ETHTOOL_A_PATH_2 = """
sos_commands/networking/ethtool_-a_g_bond2
""".strip()

SUCCESS_ETHTOOL_A_BLANK_LINE = """
Pause parameters for enp0s25:

Autonegotiate : on
RX:             off
TX:             off
RX negotiated:  off
TX negotiated:  off
""".strip()

ETHTOOL_T_NO_CAP_VALUE = """
Time stamping parameters for ens192:
Capabilities:
    software-receive
    software-system-clock
PTP Hardware Clock: none
Hardware Transmit Timestamp Modes: none
Hardware Receive Filter Modes: none
"""


def test_ethtool_a():
    context = context_wrap(SUCCESS_ETHTOOL_A, path=SUCCESS_ETHTOOL_A_PATH)
    result = ethtool.Pause(context)
    assert result.ifname == "enp0s25"
    assert result.autonegotiate
    assert not result.rx
    assert not result.tx
    assert not result.rx_negotiated
    assert not result.tx_negotiated


def test_ethtool_a_1():
    context = context_wrap(FAIL_ETHTOOL_A, path=FAIL_ETHTOOL_A_PATH)
    result = ethtool.Pause(context)
    assert result.ifname == "__wlp3s0"
    assert not result.autonegotiate


def test_ethtool_a_2():
    context = context_wrap(FAIL_ETHTOOL_A_1, path=FAIL_ETHTOOL_A_PATH_1)
    result = ethtool.Pause(context)
    assert result.ifname == "bond0.1384@bond0"


def test_ethtool_a_3():
    context = context_wrap(FAIL_ETHTOOL_A_2, path=FAIL_ETHTOOL_A_PATH_2)
    result = ethtool.Pause(context)
    assert result.ifname == "g_bond2"


def test_ethtool_a_blank_line():
    context = context_wrap(SUCCESS_ETHTOOL_A_BLANK_LINE, path=SUCCESS_ETHTOOL_A_PATH)
    result = ethtool.Pause(context)
    assert result.ifname == "enp0s25"


TEST_ETHTOOL_C_DOCS = '''
Coalesce parameters for eth0:
Adaptive RX: off  TX: off
stats-block-usecs: 0
sample-interval: 0
pkt-rate-low: 0
pkt-rate-high: 0

rx-usecs: 20
rx-frames: 5
rx-usecs-irq: 0
rx-frames-irq: 5

tx-usecs: 72
tx-frames: 53
tx-usecs-irq: 0
tx-frames-irq: 5

rx-usecs-low: 0
rx-frame-low: 0
tx-usecs-low: 0
tx-frame-low: 0

rx-usecs-high: 0
rx-frame-high: 0
tx-usecs-high: 0
tx-frame-high: 0
'''

TEST_ETHTOOL_C = """
Coalesce parameters for eth2:
Adaptive RX: off  TX: off
pkt-rate-high: 10

tx-usecs-irq: 0
tx-frame-low: 25

tx-usecs-high: 0
tx-frame-high: n/a

""".strip()

TEST_ETHTOOL_C_PATH = "sos_commands/networking/ethtool_-c_eth2"

TEST_ETHTOOL_C_CANNOT_GET = """
Cannot get device coalesce settings: Operation not supported
Coalesce parameters for usb0:
""".strip()

TEST_ETHTOOL_C_BAD_ARGS = """
ethtool: bad command line argument(s)
For more information run ethtool -h
"""

TEST_ETHTOOL_C_SHORT = """
Coalesce parameters for eth2:
""".strip()


def test_get_ethtool_c():
    context = context_wrap(TEST_ETHTOOL_C)
    context.path = TEST_ETHTOOL_C_PATH
    result = ethtool.CoalescingInfo(context)
    assert keys_in(["adaptive-rx", "adaptive-tx", "pkt-rate-high",
                    "tx-usecs-irq", "tx-frame-low", "tx-usecs-high"], result.data)
    assert result.ifname == "eth2"
    assert not result.adaptive_rx
    assert not result.adaptive_tx
    assert result.pkt_rate_high == 10
    assert result.tx_usecs_irq == 0
    assert result.tx_frame_low == 25
    assert result.tx_usecs_high == 0
    assert 'tx_frame_high' not in result.data


def test_get_ethtool_c_cannot_get():
    context = context_wrap(TEST_ETHTOOL_C_CANNOT_GET)
    context.path = 'sos_commands/networking/ethtool_-c_usb0'
    result = ethtool.CoalescingInfo(context)
    assert result.ifname == 'usb0'


def test_get_ethtool_c_bad_args():
    context = context_wrap(TEST_ETHTOOL_C_BAD_ARGS, path='sos_commands/networking/ethtool_-c_eth0')
    result = ethtool.CoalescingInfo(context)
    assert result.ifname == 'eth0'


def test_get_ethtool_c_short():
    context = context_wrap(TEST_ETHTOOL_C_SHORT, path=TEST_ETHTOOL_C_PATH)
    with pytest.raises(ParseException) as exc:
        ethtool.CoalescingInfo(context)
    assert 'Command output missing value data' in str(exc)


TEST_ETHTOOL_G_DOCS = '''
Ring parameters for eth0:
Pre-set maximums:
RX:             2047
RX Mini:        0
RX Jumbo:       0
TX:             511
Current hardware settings:
RX:             200
RX Mini:        0
RX Jumbo:       0
TX:             511
'''

TEST_ETHTOOL_G = """
Ring parameters for eth2:
Pre-set maximums:
RX:     2047
RX Mini:    0
RX Jumbo:   0
TX:     511

Current hardware settings:
RX:     200
RX Mini:    0
RX Jumbo:   0
TX:     511


""".strip()

TEST_ETHTOOL_G_PATH = """
sos_commands/networking/ethtool_-g_eth2
""".strip()

TEST_ETHTOOL_G_1 = """
Cannot get device ring settings: No such device
Ring parameters for bond0.102@bond0:

"""

TEST_ETHTOOL_G_PATH_1 = """
sos-command/neworking/ethtool_-g_bond2.102@bond2
"""

TEST_ETHTOOL_G_2 = """
ethtool: bad command line argument(s)
For more information run ethtool -h
"""

TEST_ETHTOOL_G_PATH_2 = """
sos_commands/networkking/ethtool_-g_eth2
"""

TEST_ETHTOOL_G_3 = """
Ring parameters for ens3:
Pre-set maximums:
RX:		256
RX Mini:	n/a
RX Jumbo:	n/a
TX:		256
Current hardware settings:
RX:		256
RX Mini:	n/a
RX Jumbo:	n/a
TX:		256
""".strip()

TEST_ETHTOOL_G_PATH_3 = """
sos_commands/networkking/ethtool_-g_ens3
""".strip()

TEST_ETHTOOL_G_4 = """
Ring parameters for ens3:
Pre-set maximums:
RX:		256
RX Mini:	some-unexpected-value
RX Jumbo:	some-unexpected-value
TX:		256
Current hardware settings:
RX:		256
RX Mini:	some-unexpected-value
RX Jumbo:	some-unexpected-value
TX:		256
""".strip()

TEST_ETHTOOL_G_PATH_4 = """
sos_commands/networkking/ethtool_-g_ens3
""".strip()

TEST_ETHTOOL_G_5 = """
Ring parameters for pri:
Pre-set maximums:
RX:		4096
RX Mini:	2048
RX Jumbo:	4096
TX:		4096
Current hardware settings:
RX:		1024
RX Mini:	128
RX Jumbo:	512
TX:		512
RX Buf Len:	n/a
CQE Size:	n/a
TX Push:	off
TCP data split:	n/a
""".strip()

TEST_ETHTOOL_G_PATH_5 = """
sos_commands/networkking/ethtool_-g_pri
""".strip()


def test_ethtool_g():
    context = context_wrap(TEST_ETHTOOL_G)
    context.path = TEST_ETHTOOL_G_PATH
    result = ethtool.Ring(context)
    assert keys_in(["max", "current"], result.data)

    assert result.ifname == "eth2"
    assert result.data['max'].rx == 2047
    assert result.data['max'].rx_mini == 0
    assert result.data['max'].rx_jumbo == 0
    assert result.data['max'].tx == 511
    assert result.max.rx == 2047
    assert result.max.rx_mini == 0
    assert result.max.rx_jumbo == 0
    assert result.max.tx == 511
    assert result.current.rx == 200
    assert result.current.rx_mini == 0
    assert result.current.rx_jumbo == 0
    assert result.current.tx == 511


def test_ethtool_g_1():
    context = context_wrap(TEST_ETHTOOL_G_1)
    context.path = TEST_ETHTOOL_G_PATH_1
    result = ethtool.Ring(context)
    assert result.ifname == "bond0.102@bond0"


def test_ethtool_g_2():
    context = context_wrap(TEST_ETHTOOL_G_2, path=TEST_ETHTOOL_G_PATH_2)
    result = ethtool.Ring(context)
    assert result.ifname == "eth2"


def test_ethtool_g_3():
    context = context_wrap(TEST_ETHTOOL_G_3, path=TEST_ETHTOOL_G_PATH_3)
    result = ethtool.Ring(context)
    assert keys_in(["max", "current"], result.data)
    assert result.ifname == "ens3"
    assert result.data['max'].rx == 256
    assert result.data['max'].rx_mini == -1
    assert result.current.rx_jumbo == -1
    assert result.current.tx == 256


def test_ethtool_g_4():
    context = context_wrap(TEST_ETHTOOL_G_4, path=TEST_ETHTOOL_G_PATH_4)
    result = ethtool.Ring(context)
    assert keys_in(["max", "current"], result.data)
    assert result.ifname == "ens3"
    assert result.data['max'].rx == 256
    assert result.data['max'].rx_mini == -2
    assert result.current.rx_jumbo == -2
    assert result.current.tx == 256


def test_ethtool_g_5():
    context = context_wrap(TEST_ETHTOOL_G_5, path=TEST_ETHTOOL_G_PATH_5)
    result = ethtool.Ring(context)
    assert keys_in(["max", "current", "current_extra"], result.data)
    assert result.ifname == "pri"
    assert result.data['max'].rx == 4096
    assert result.data['max'].rx_mini == 2048
    assert result.current.rx_jumbo == 512
    assert result.current.tx == 512
    assert result.current_extra == {
        'rx_buf_len': -1, 'cqe_size': -1, 'tx_push': -2, 'tcp_data_split': -1}
    assert result.current_extra.get("rx_buf_len") == -1
    assert result.current_extra["tx_push"] == -2


TEST_ETHTOOL_I_DOCS = '''
driver: bonding
version: 3.6.0
firmware-version: 2
bus-info:
supports-statistics: no
supports-test: no
supports-eeprom-access: no
supports-register-dump: no
supports-priv-flags: no
'''

TEST_ETHTOOL_I_GOOD = """
driver: virtio_net
version: 1.0.0
firmware-version:
bus-info: 0000:00:03.0
supports-statistics: no
supports-test: no
supports-eeprom-access: no
supports-register-dump: no
supports-priv-flags: no
"""

TEST_ETHTOOL_I_MISSING_KEYS = """
driver: virtio_net
firmware-version:
bus-info: 0000:00:03.0
supports-statistics: no
supports-test: no
supports-eeprom-access: no
supports-register-dump: no
supports-priv-flags: no
something without a colon here is ignored
"""


def test_good():
    context = context_wrap(TEST_ETHTOOL_I_GOOD)
    parsed = ethtool.Driver(context)
    assert parsed.version == "1.0.0"
    assert parsed.driver == 'virtio_net'
    assert parsed.firmware_version is None
    assert parsed.bus_info == '0000:00:03.0'
    assert not parsed.supports_statistics
    assert not parsed.supports_test
    assert not parsed.supports_eeprom_access
    assert not parsed.supports_register_dump
    assert not parsed.supports_priv_flags


def test_missing_version():
    context = context_wrap(TEST_ETHTOOL_I_MISSING_KEYS)
    parsed = ethtool.Driver(context)
    assert parsed.version is None


def test_missing_value():
    context = context_wrap(TEST_ETHTOOL_I_GOOD)
    parsed = ethtool.Driver(context)
    assert parsed.firmware_version is None


def test_iface():
    context = context_wrap(TEST_ETHTOOL_I_GOOD, path="sbin/ethtool_-i_eth0")
    parsed = ethtool.Driver(context)
    assert parsed.ifname == "eth0"


def test_no_iface():
    context = context_wrap(TEST_ETHTOOL_I_GOOD, path="foo/bar/baz/oopsie")
    parsed = ethtool.Driver(context)
    assert parsed.ifname is None


ETHTOOL_K_STANDARD = """
Features for bond0:
rx-checksumming: off [fixed]
tx-checksumming: on
    tx-checksum-ipv4: off [fixed]
    tx-checksum-unneeded: on [fixed]
    tx-checksum-ip-generic: off [fixed]
    tx-checksum-ipv6: off [fixed]
    tx-checksum-fcoe-crc: off [fixed]
    tx-checksum-sctp: off [fixed]
scatter-gather: on
    tx-scatter-gather: on [fixed]
    tx-scatter-gather-fraglist: off [fixed]
tcp-segmentation-offload: on
    tx-tcp-segmentation: on [fixed]
    tx-tcp-ecn-segmentation: on [fixed]
    tx-tcp6-segmentation: on [fixed]
udp-fragmentation-offload: off [fixed]
generic-segmentation-offload: off [requested on]
generic-receive-offload: on
large-receive-offload: on
rx-vlan-offload: on
tx-vlan-offload: on
ntuple-filters: off
receive-hashing: off
highdma: on [fixed]
rx-vlan-filter: on [fixed]
vlan-challenged: off [fixed]
tx-lockless: on [fixed]
netns-local: off [fixed]
tx-gso-robust: off [fixed]
tx-fcoe-segmentation: off [fixed]
tx-gre-segmentation: on [fixed]
tx-udp_tnl-segmentation: on [fixed]
fcoe-mtu: off [fixed]
loopback: off [fixed]
"""

ETHTOOL_K_BAD_ARGS = """
ethtool: bad command line argument(s)
For more information run ethtool -h
""".strip()

ETHTOOL_K_CANNOT_GET = """
Cannot get stats strings information: Operation not supported
""".strip()

ETHTOOL_K_MISSING_COLON = """
Features for bond0:
rx-checksumming off [fixed]
tx-checksumming on
"""


def test_Features_good():
    # Test picking up interface from header line rather than path
    features = ethtool.Features(
        context_wrap(ETHTOOL_K_STANDARD, path='sos_commands/networking/ethtool_-k_bond0'))
    assert features.ifname == 'bond0'
    assert features.iface == 'bond0'

    assert not features.is_on('rx-checksumming')
    assert features.is_on('tx-checksumming')

    assert features.is_fixed('rx-checksumming')
    assert not features.is_fixed('tx-checksumming')


def test_Features_bad_args():
    # Even if content is bad, interface should be available from path
    features = ethtool.Features(
        context_wrap(ETHTOOL_K_BAD_ARGS, path="sbin/ethtool_-k_bond0"))
    assert features.ifname == 'bond0'
    assert features.iface == 'bond0'
    assert features.data == {}


def test_Features_cannot_get():
    features = ethtool.Features(
        context_wrap(ETHTOOL_K_CANNOT_GET, path="sbin/ethtool_-k_eth1"))
    assert features.ifname == 'eth1'
    assert features.iface == 'eth1'

    assert features.data == {}


def test_Features_missing_colon():
    features = ethtool.Features(
        context_wrap(ETHTOOL_K_MISSING_COLON, path="sbin/ethtool_-k_bond0"))
    assert features.ifname == 'bond0'
    assert features.data == {}


TEST_ETHTOOL_S_DOCS = '''
NIC statistics:
     rx_octets: 808488730
     rx_fragments: 0
     rx_ucast_packets: 1510830
     rx_mcast_packets: 678653
     rx_bcast_packets: 9921
     rx_fcs_errors: 0
     rx_align_errors: 0
     rx_xon_pause_rcvd: 0
     rx_xoff_pause_rcvd: 0
     rx_mac_ctrl_rcvd: 0
     rx_xoff_entered: 0
     rx_frame_too_long_errors: 0
     rx_jabbers: 0
'''

SUCCEED_ETHTOOL_S = '''
NIC statistics:
    rx_packets: 912398
    tx_packets: 965449
    rx_bytes: 96506134
    tx_bytes: 190360255
    rx_broadcast: 4246
    tx_broadcast: 4248
    rx_multicast: 18
    tx_multicast: 20
    multicast: 18
    collisions: 0
    rx_crc_errors: 0
    rx_no_buffer_count: 0
    rx_missed_errors: 0
    tx_aborted_errors: 0
    tx_carrier_errors: 0
    tx_window_errors: 0
    tx_abort_late_coll: 0
    tx_deferred_ok: 0
    tx_single_coll_ok: 0
    tx_multi_coll_ok: 0
    tx_timeout_count: 0
    rx_long_length_errors: 0
    rx_short_length_errors: 0
    rx_align_errors: 0
    tx_tcp_seg_good: 0
    tx_tcp_seg_failed: 0
    rx_flow_control_xon: 0
    rx_flow_control_xoff: 0
    tx_flow_control_xon: 0
    tx_flow_control_xoff: 0
    rx_long_byte_count: 96506134
    tx_dma_out_of_sync: 0
    tx_smbus: 0
    rx_smbus: 0
    dropped_smbus: 0
    os2bmc_rx_by_bmc: 0
    os2bmc_tx_by_bmc: 0
    os2bmc_tx_by_host: 0
    os2bmc_rx_by_host: 0
    tx_hwtstamp_timeouts: 0
    rx_hwtstamp_cleared: 0
    rx_errors: 0
    tx_errors: 0
    tx_dropped: 0
    rx_length_errors: 0
    rx_over_errors: 0
    rx_frame_errors: 0
    rx_fifo_errors: 0
    tx_fifo_errors: 0
    tx_heartbeat_errors: 0
    tx_queue_0_packets: 613
    tx_queue_0_bytes: 240342
    tx_queue_0_restart: 0
    tx_queue_1_packets: 935473
    tx_queue_1_bytes: 181899495
    tx_queue_1_restart: 0
    tx_queue_2_packets: 6
    tx_queue_2_bytes: 412
    tx_queue_2_restart: 0
    tx_queue_3_packets: 29357
    tx_queue_3_bytes: 4358198
    tx_queue_3_restart: 0
    rx_queue_0_packets: 912398
    rx_queue_0_bytes: 92856542
    rx_queue_0_drops: 0
    rx_queue_0_csum_err: 0
    rx_queue_0_alloc_failed: 0
    rx_queue_1_packets: 0
    rx_queue_1_bytes: 0
    rx_queue_1_drops: 0
    rx_queue_1_csum_err: 0
    rx_queue_1_alloc_failed: 0
    rx_queue_2_packets: 0
    rx_queue_2_bytes: 0
    rx_queue_2_drops: 0
    rx_queue_2_csum_err: 0
    rx_queue_2_alloc_failed: 0
    rx_queue_3_packets: 0
    rx_queue_3_bytes: 0
    rx_queue_3_drops: 0
    rx_queue_3_csum_err: 0
    rx_queue_3_alloc_failed: 0
'''

ETHTOOL_S_eth0_1 = """NIC statistics:
    rxq0: rx_pkts: 5000000
    rxq0: rx_drops_no_frags: 5000
    rxq1: rx_pkts: 5000000
    rxq1: rx_drops_no_frags: 5000
"""

FAILED_ETHTOOL_S_ONE = "no stats avilable "

FAILED_ETHTOOL_S_TWO = "Cannot get stats strings information: Operation not supported"

FAILED_ETHTOOL_S_THREE = """NIC statistics:
Nothing to see here
"""


def test_ethtool_S():
    ethtool_S_info = ethtool.Statistics(context_wrap(SUCCEED_ETHTOOL_S))
    ret = {}
    for line in SUCCEED_ETHTOOL_S.split('\n')[2:-1]:
        key, value = line.split(':')
        ret[key.strip()] = int(value.strip()) if value else None
    eth_data = dict(ethtool_S_info.data)
    assert eth_data == ret

    # Test search functionality
    assert ethtool_S_info.search(r'rx_queue_3') == {
        'rx_queue_3_packets': 0,
        'rx_queue_3_bytes': 0,
        'rx_queue_3_drops': 0,
        'rx_queue_3_csum_err': 0,
        'rx_queue_3_alloc_failed': 0,
    }
    assert ethtool_S_info.search(r'RX_QUEUE_3', flags=re.IGNORECASE) == {
        'rx_queue_3_packets': 0,
        'rx_queue_3_bytes': 0,
        'rx_queue_3_drops': 0,
        'rx_queue_3_csum_err': 0,
        'rx_queue_3_alloc_failed': 0,
    }


def test_ethtool_S_subinterface():
    ethtool_S_info = ethtool.Statistics(context_wrap(ETHTOOL_S_eth0_1))
    assert sorted(ethtool_S_info.data.keys()) == sorted([
        'rxq0: rx_pkts', 'rxq0: rx_drops_no_frags',
        'rxq1: rx_pkts', 'rxq1: rx_drops_no_frags'
    ])
    assert ethtool_S_info.data['rxq0: rx_pkts'] == 5000000


def test_ethtool_S_f():
    ethtool_S_info_f1 = ethtool.Statistics(context_wrap(FAILED_ETHTOOL_S_ONE))
    assert not ethtool_S_info_f1.ifname
    ethtool_S_info_f2 = ethtool.Statistics(context_wrap(FAILED_ETHTOOL_S_TWO))
    assert not ethtool_S_info_f2.ifname
    ethtool_S_info_f2 = ethtool.Statistics(context_wrap(FAILED_ETHTOOL_S_THREE))
    assert not ethtool_S_info_f2.ifname


TEST_ETHTOOL_TIMESTAMP = '''
Time stamping parameters for eno1:

Capabilities:
    hardware-transmit     (SOF_TIMESTAMPING_TX_HARDWARE)
    software-transmit     (SOF_TIMESTAMPING_TX_SOFTWARE)
    hardware-receive      (SOF_TIMESTAMPING_RX_HARDWARE)
    software-receive      (SOF_TIMESTAMPING_RX_SOFTWARE)
    software-system-clock (SOF_TIMESTAMPING_SOFTWARE)
    hardware-raw-clock    (SOF_TIMESTAMPING_RAW_HARDWARE)
PTP Hardware Clock: 0
Hardware Transmit Timestamp Modes:
    off                   (HWTSTAMP_TX_OFF)
    on                    (HWTSTAMP_TX_ON)
Hardware Receive Filter Modes:
    none                  (HWTSTAMP_FILTER_NONE)
    all                   (HWTSTAMP_FILTER_ALL)
'''

TEST_ETHTOOL_TIMESTAMP_AB = '''
Time stamping parameters for eno1:

Capabilities:
    hardware-transmit     (SOF_TIMESTAMPING_TX_HARDWARE
    software-transmit     (SOF_TIMESTAMPING_TX_SOFTWARE)
    hardware-receive      (SOF_TIMESTAMPING_RX_HARDWARE)
    software-receive      (SOF_TIMESTAMPING_RX_SOFTWARE)
    software-system-clock (SOF_TIMESTAMPING_SOFTWARE)
    hardware-raw-clock    (SOF_TIMESTAMPING_RAW_HARDWARE)
PTP Hardware Clock: 0
Hardware Transmit Timestamp Modes:
    off                   (HWTSTAMP_TX_OFF)
    on                    (HWTSTAMP_TX_ON)
Hardware Receive Filter Modes:
    none                  (HWTSTAMP_FILTER_NONE)
    all                   (HWTSTAMP_FILTER_ALL)
'''

ETHTOOL_T_NO_CAP_VALUE = """
Time stamping parameters for ens192:
Capabilities:
    software-receive
    software-system-clock
PTP Hardware Clock: none
Hardware Transmit Timestamp Modes: none
Hardware Receive Filter Modes: none
"""


def test_ethtool_timestamp():
    timestamp = ethtool.TimeStamp(context_wrap(TEST_ETHTOOL_TIMESTAMP, path="sbin/ethtool_-T_eno1"))
    assert timestamp.ifname == 'eno1'
    assert timestamp.data['Capabilities']['hardware-transmit'] == 'SOF_TIMESTAMPING_TX_HARDWARE'
    assert timestamp.data['Capabilities']['hardware-raw-clock'] == 'SOF_TIMESTAMPING_RAW_HARDWARE'
    assert timestamp.data['PTP Hardware Clock'] == '0'
    assert timestamp.data['Hardware Transmit Timestamp Modes']['off'] == 'HWTSTAMP_TX_OFF'
    assert timestamp.data['Hardware Receive Filter Modes']['all'] == 'HWTSTAMP_FILTER_ALL'


def test_ethtool_timestamp_no_cap_value():
    timestamp = ethtool.TimeStamp(context_wrap(ETHTOOL_T_NO_CAP_VALUE, path="sbin/ethtool_-T_ens192"))
    assert timestamp.ifname == 'ens192'
    assert timestamp.data['Capabilities']['software-receive'] is None
    assert timestamp.data['Hardware Transmit Timestamp Modes'] == 'none'


TEST_EXTRACT_FROM_PATH_1 = """
    ethtool_-a_eth0
""".strip()
TEST_EXTRACT_FROM_PATH_2 = """
    ethtool_-a_bond0.104_bond0
""".strip()
TEST_EXTRACT_FROM_PATH_3 = """
    ethtool_-a___tmp199222
""".strip()
TEST_EXTRACT_FROM_PATH_4 = """
    ethtool_-a_macvtap_bond0
""".strip()
TEST_EXTRACT_FROM_PATH_5 = """
    ethtool_-a_p3p2.2002-fcoe_p3p2
""".strip()
TEST_EXTRACT_FROM_PATH_PARAM = """
    ethtool_-a
""".strip()


def test_extract_from_path_1():
    test_data = [
        (TEST_EXTRACT_FROM_PATH_1, "eth0"),
        (TEST_EXTRACT_FROM_PATH_2, "bond0.104@bond0"),
        (TEST_EXTRACT_FROM_PATH_3, "__tmp199222"),
        (TEST_EXTRACT_FROM_PATH_4, "macvtap@bond0"),
        (TEST_EXTRACT_FROM_PATH_5, "p3p2.2002-fcoe@p3p2"),
    ]
    for input_, value in test_data:
        ifname = ethtool.extract_iface_name_from_path(input_, TEST_EXTRACT_FROM_PATH_PARAM)
        assert ifname == value


TEST_ETHTOOL_DOCS = '''
        Settings for eth0:
                Supported ports: [ TP MII ]
                Supported link modes:   10baseT/Half 10baseT/Full
                                       100baseT/Half 100baseT/Full
                Supported pause frame use: No
                Supports auto-negotiation: Yes
                Advertised link modes:  10baseT/Half 10baseT/Full
                                        100baseT/Half 100baseT/Full
                Advertised pause frame use: Symmetric
                Advertised auto-negotiation: Yes
                Link partner advertised link modes:  10baseT/Half 10baseT/Full
                                                     100baseT/Half 100baseT/Full
                Link partner advertised pause frame use: Symmetric
                Link partner advertised auto-negotiation: No
                Speed: 100Mb/s
                Duplex: Full
                Port: MII
                PHYAD: 32
                Transceiver: internal
                Auto-negotiation: on
        Cannot get wake-on-lan settings: Operation not permitted
                Current message level: 0x00000007 (7)
                                       drv probe link
        Cannot get link status: Operation not permitted
'''

ETHTOOL_INFO = """
Cannot get wake-on-lan settings: Operation not permitted
Cannot get link status: Operation not permitted
Settings for eth1:
    Supported ports: [ TP MII ]
    Supported link modes: 10baseT/Half 10baseT/Full
                          100baseT/Half 100baseT/Full
                          1000baseT/Full
    Supported pause frame use: Symmetric
    Supports auto-negotiation: Yes
    Advertised link modes: 10baseT/Half 10baseT/Full
                           100baseT/Half 100baseT/Full
                           1000baseT/Full
    Advertised pause frame use: Symmetric
    Advertised auto-negotiation: Yes
    Speed: 1000Mb/s
    Duplex: Full
    Port: Twisted Pair
    PHYAD: 1
    Transceiver: internal
    Auto-negotiation: on
    MDI-X: off (auto)
    Supports Wake-on: pumbg
    Wake-on: d
    Current message level: 0x00000007 (7)
                           drv probe link
    Link detected: yes
""".strip()

ETHTOOL_INFO_TEST = """
Settings for eth1:
    Supported pause frame use: Symmetric

    Supports auto-negotiation: Yes
""".strip()


def test_ethtool():
    ethtool_info = ethtool.Ethtool(context_wrap(ETHTOOL_INFO, path="ethtool_eth1"))
    assert ethtool_info.ifname == "eth1"
    assert ethtool_info.link_detected
    assert ethtool_info.speed == ['1000Mb/s']
    assert ethtool_info.supported_link_modes == [
        '10baseT/Half', '10baseT/Full',
        '100baseT/Half', '100baseT/Full',
        '1000baseT/Full'
    ]
    assert ethtool_info.advertised_link_modes == [
        '10baseT/Half', '10baseT/Full',
        '100baseT/Half', '100baseT/Full',
        '1000baseT/Full'
    ]
    assert ethtool_info.supported_ports == ['TP', 'MII']


def test_ethtool_corner_cases():
    ethtool_info = ethtool.Ethtool(context_wrap(ETHTOOL_INFO_TEST, path="ethtool_eth1"))
    assert ethtool_info.ifname == "eth1"


ETHTOOL_INFO_BAD_1 = """
Settings for dummy2:
No data available
""".strip()

ETHTOOL_INFO_BAD_2 = """
No data available
""".strip()

ETHTOOL_INFO_BAD_3 = """
Settings for eth1:
    Supported ports
    Supported link modes 10baseT/Half 10baseT/Full
                          100baseT/Half 100baseT/Full
"""


def test_ethtool_fail():
    with pytest.raises(ParseException):
        ethtool.Ethtool(context_wrap(FAIL_ETHTOOL_A_1, path="ethtool_eth1"))
    with pytest.raises(ParseException) as e:
        ethtool.Ethtool(context_wrap(ETHTOOL_INFO_BAD_1, path="ethtool_eth1"))
    assert "Fake ethnic as ethtool command argument" in str(e.value)

    with pytest.raises(ParseException) as e:
        ethtool.Ethtool(context_wrap(ETHTOOL_INFO_BAD_2, path="ethtool_eth1"))
    assert "Ethtool does not contain Settings for <nic>:" in str(e.value)

    with pytest.raises(ParseException) as e:
        ethtool.Ethtool(context_wrap(ETHTOOL_INFO_BAD_3, path="ethtool_eth1"))
    assert 'Ethtool unable to parse content' in str(e.value)


# Because tests are done at the module level, we have to put all the shared
# parser information in the one environment.  Fortunately this is normal.
# Also note that to support common usage, all of these are supplied in lists.
def test_ethtool_i_doc_examples():
    env = {
        'coalesce': [ethtool.CoalescingInfo(context_wrap(TEST_ETHTOOL_C_DOCS, path='ethtool_-c_eth0'))],
        'interfaces': [ethtool.Driver(context_wrap(TEST_ETHTOOL_I_DOCS, path='ethtool_-i_bond0'))],
        'ethers': [ethtool.Ethtool(context_wrap(TEST_ETHTOOL_DOCS, path='ethtool_eth0'))],
        'features': [ethtool.Features(context_wrap(ETHTOOL_K_STANDARD, path='ethtool_-k_bond0'))],
        'pause': [ethtool.Pause(context_wrap(TEST_ETHTOOL_A_DOCS, path='ethtool_-a_eth0'))],
        'ring': [ethtool.Ring(context_wrap(TEST_ETHTOOL_G_DOCS, path='ethtool_-g_eth0'))],
        'stats': [ethtool.Statistics(context_wrap(TEST_ETHTOOL_S_DOCS, path='ethtool_-S_eth0'))],
        'timestamp': [ethtool.TimeStamp(context_wrap(TEST_ETHTOOL_TIMESTAMP, path='ethtool_-T_eno1'))],
    }
    failed, total = doctest.testmod(ethtool, globs=env)
    assert failed == 0



import os

from mock.mock import patch, Mock
from pytest import mark

from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.core.spec_cleaner import Cleaner

test_file_data = 'test\nabcd\n1234\npwd: p4ssw0rd\n'
test_file_data_sensitive = 'test\nabcd\n1234\npassword: p4ssw0rd\n'


@patch('insights.client.archive.InsightsArchive', Mock())
@patch('insights.client.insights_spec.InsightsCommand', Mock())
@patch('insights.client.insights_spec.InsightsFile', Mock())
@patch('insights.client.data_collector.DataCollector._parse_command_spec', Mock())
@patch('insights.client.data_collector.DataCollector._parse_file_spec', Mock())
@patch('insights.client.data_collector.DataCollector._parse_glob_spec', Mock())
def test_redact_classic():
    '''
    Verify redact is filled during classic collection
    '''
    conf = InsightsConfig()
    rm_conf = {'test': 'test'}
    pp = Cleaner(conf, rm_conf)
    assert pp.redact['exclude'] == []
    assert pp.redact['regex'] is False


@patch('insights.client.archive.InsightsArchive', Mock())
@patch('insights.client.core_collector.CoreCollector._write_branch_info', Mock())
@patch('insights.client.core_collector.CoreCollector._write_display_name', Mock())
@patch('insights.client.core_collector.CoreCollector._write_version_info', Mock())
@patch('insights.client.core_collector.CoreCollector._write_tags', Mock())
@patch('insights.client.core_collector.CoreCollector._write_blacklist_report', Mock())
@patch('insights.client.core_collector.collect.collect', Mock(return_value=('/var/tmp/testarchive/insights-test', {})))
def test_redact_core():
    conf = InsightsConfig(core_collect=True)
    rm_conf = {'test': 'test'}
    pp = Cleaner(conf, rm_conf)
    assert pp.redact['exclude'] == []
    assert pp.redact['regex'] is False


@mark.parametrize("obfuscate", [True, False])
@mark.parametrize("core_collect", [True, False])
def test_redact_line_changed_password(core_collect, obfuscate):
    conf = InsightsConfig(core_collect=core_collect, obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data_sensitive)

    pp = Cleaner(conf, {})
    pp.clean_file(test_file, [])
    # file is changed
    with open(test_file, 'r') as t:
        data = t.readlines()
        assert 'p4ssw0rd' not in data[-1]
        assert '********' in data[-1]
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
@mark.parametrize("core_collect", [True, False])
def test_redact_keyword_empty_not_change(core_collect, obfuscate):
    conf = InsightsConfig(core_collect=core_collect, obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, {})  # empty keywords
    pp.clean_file(test_file, [])
    # file is NOT changed
    with open(test_file, 'r') as t:
        assert test_file_data == ''.join(t.readlines())
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
@mark.parametrize("core_collect", [True, False])
def test_redact_keyword_changed_keyword(core_collect, obfuscate):
    conf = InsightsConfig(core_collect=core_collect, obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, {'keywords': ['test']})
    pp.clean_file(test_file, [])
    # file is changed
    with open(test_file, 'r') as t:
        data = t.readlines()
        assert 'test' not in data[0]
        assert 'keyword0' in data[0]
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
@mark.parametrize("core_collect", [True, False])
def test_redact_keyword_no_such_keyword_to_change(core_collect, obfuscate):
    conf = InsightsConfig(core_collect=core_collect, obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, {'keywords': ['t_e_s_t']})  # no such keyword
    pp.clean_file(test_file, [])
    # file is NOT changed
    with open(test_file, 'r') as t:
        assert test_file_data == ''.join(t.readlines())
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
@mark.parametrize("core_collect", [True, False])
def test_redact_keyword_disabled_by_no_redact(core_collect, obfuscate):
    conf = InsightsConfig(core_collect=core_collect, obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, {'keywords': 'test'})
    pp.clean_file(test_file, [], no_redact=True)
    # file is NOT changed
    with open(test_file, 'r') as t:
        assert test_file_data == ''.join(t.readlines())
    arch.delete_archive_dir()


@mark.parametrize(("line", "expected"), [
    (
        "what's your name? what day is today?",
        "what's your keyword0? what keyword1 is tokeyword1?"
    ),
])
@mark.parametrize("obfuscate", [True, False])
@mark.parametrize("core_collect", [True, False])
def test_redact_line_keyword_extract_cases(core_collect, obfuscate, line, expected):
    conf = InsightsConfig(core_collect=core_collect, obfuscate=obfuscate)
    pp = Cleaner(conf, {'keywords': ['name', 'day']})
    actual = pp._redact_line(line)
    assert actual == expected


@mark.parametrize("core_collect", [True, False])
def test_redact_line_keyword_with_hostname_and_ip(core_collect):
    hostname = 'test1.abc.com'
    line = "test1.abc.com, 10.0.0.1, test1.abc.loc, 20.1.4.7, smtp.abc.com, what's your name?, what day is today?"
    conf = InsightsConfig(core_collect=core_collect, obfuscate=True, obfuscate_hostname=True, hostname=hostname)
    pp = Cleaner(conf, {'keywords': ['name', 'day']}, hostname)
    result = pp._redact_line(line)
    assert 'test1.abc.com' in result  # hostname is not processed in _redact_line
    assert '10.0.0.1' in result  # IP is not processed in _redact_line
    assert '20.1.4.7' in result  # IP is not processed in _redact_line
    assert 'name' not in result
    assert 'day' not in result
    assert 'keyword0' in result
    assert 'keyword1' in result


@mark.parametrize("obfuscate", [True, False])
@mark.parametrize("core_collect", [True, False])
def test_redact_patterns_exclude_regex(core_collect, obfuscate):
    conf = InsightsConfig(core_collect=core_collect, obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    rm_conf = {'patterns': {'regex': ['12.*4', '^abcd']}}
    pp = Cleaner(conf, rm_conf)
    pp.clean_file(test_file, [])
    with open(test_file, 'r') as t:
        data = [i.strip() for i in t.readlines()]
        assert '1234' not in data
        assert 'abcd' not in data
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
@mark.parametrize("core_collect", [True, False])
def test_redact_file_empty(core_collect, obfuscate):
    conf = InsightsConfig(core_collect=core_collect, obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    rm_conf = {'patterns': {'regex': ['test', 'pwd', '12.*4', '^abcd']}}
    pp = Cleaner(conf, rm_conf)
    pp.clean_file(test_file)
    # file is cleaned to empty, hence it was removed
    assert not os.path.exists(test_file)
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
@mark.parametrize("core_collect", [True, False])
def test_redact_patterns_exclude_no_regex(core_collect, obfuscate):
    conf = InsightsConfig(core_collect=core_collect, obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    rm_conf = {'patterns': ['1234', 'abcd']}
    pp = Cleaner(conf, rm_conf)
    pp.clean_file(test_file, [])
    with open(test_file, 'r') as t:
        data = [i.strip() for i in t.readlines()]
        assert '1234' not in data
        assert 'abcd' not in data
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
@mark.parametrize("core_collect", [True, False])
def test_redact_patterns_exclude_empty(core_collect, obfuscate):
    conf = InsightsConfig(core_collect=core_collect, obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    rm_conf = {'patterns': []}
    pp = Cleaner(conf, rm_conf)
    pp.clean_file(test_file, [])
    # file is not changed
    with open(test_file, 'r') as t:
        assert test_file_data == ''.join(t.readlines())
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
@mark.parametrize("core_collect", [True, False])
def test_redact_exclude_none(core_collect, obfuscate):
    conf = InsightsConfig(core_collect=core_collect, obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, None)
    pp.clean_file(test_file, [])
    # file is not changed
    with open(test_file, 'r') as t:
        assert test_file_data == ''.join(t.readlines())
    arch.delete_archive_dir()


@mark.parametrize(("line", "expected"), [
    ("test1.abc.com: it's myserver? what is yours?", None),
    ("testabc: it's mykey? what is yours?", None),
    (
        "testabc: it's my1key? what is yours?",
        "testabc: it's my1key? what is yours?",
    ),
])
def test_redact_exclude_patterns(line, expected):
    c = InsightsConfig()
    pp = Cleaner(c, {'patterns': ['myserver', 'mykey']})
    actual = pp._redact_line(line)
    assert actual == expected


@mark.parametrize(("line", "expected"), [
    ("test.abc.com: it's myserver? what is yours?", None),
    ("testabc: it's mykey? what is yours?", None),
    ("testabc: it's my1key? what is yours?", None),
    ("test1: it's my-key? what is yours?", "test1: it's my-key? what is yours?"),
])
def test_redact_patterns_regex(line, expected):
    c = InsightsConfig()
    pp = Cleaner(c, {'patterns': {'regex': ['myserver', r'my(\w*)key']}})
    actual = pp._redact_line(line)
    assert actual == expected


@mark.parametrize(("line", "expected"), [
    ("test.abc.com: it's myserver? what is yours?", None),
    ("testabc: it's mykey? what is yours?", None),
    ("testabc: it's my1key? what is yours?", None),
    ("test1: it's my-key? what is yours?", None),
    ("test: it's my-key? what is yours?", "test: it's my-key? what is yours?"),
])
def test_redact_patterns_posix_regex(line, expected):
    c = InsightsConfig()
    pp = Cleaner(c, {'patterns': {'regex': ['myserver', r'my(\w*)key', 'test[[:digit:]]']}})
    actual = pp._redact_line(line)
    assert actual == expected


@mark.parametrize(("line", "expected"), [
    ("password: p@ss_W0rd ?", "password: ******** ?"),
    ("password = p@ss_W0rd ?", "password = ******** ?"),
    ("password=p@ss_W0-d", "password=********"),
])
def test_redact_password(line, expected):
    c = InsightsConfig()
    pp = Cleaner(c, {'patterns': {'regex': ['myserver', r'my(\w*)key']}})
    actual = pp._redact_line(line)
    assert actual == expected



"""
InsightsClientConf - file ``/etc/insights-client/insights-client.conf``
=======================================================================
"""
from insights.core import IniConfigFile, JSONParser
from insights.core.filters import add_filter
from insights.core.plugins import parser
from insights.specs import Specs

# "ros_collect" is added for troubleshooting convenience
add_filter(Specs.insights_client_conf, ["[", "ros_collect"])


@parser(Specs.insights_client_conf)
class InsightsClientConf(IniConfigFile):
    """
    This class provides parsing for the file ``/etc/insights-client/insights-client.conf``.

    Sample input data is in the format::

        [insights-client]
        # Change log level, valid options DEBUG, INFO, WARNING, ERROR, CRITICAL. Default DEBUG
        loglevel=INFO
        # Log each line executed
        trace=False
        # Attempt to auto configure with Satellite server
        auto_config=True
        # Automatically update the dynamic configuration
        auto_update=False
        # Obfuscate IP addresses
        obfuscate=False

    Examples:
        >>> type(conf)
        <class 'insights.parsers.insights_client_conf.InsightsClientConf'>
        >>> conf.sections()
        ['insights-client']
        >>> conf.get("insights-client", "loglevel")
        'INFO'
        >>> conf.getboolean("insights-client", "obfuscate")
        False
    """
    pass


@parser(Specs.basic_auth_insights_client)
class BasicAuthInsightsClient(JSONParser):
    """
    Class for parsing the content of ``basic_conf``.

    Typical content::

        {"username_set": true, "pass_set": true}

    Examples:
        >>> type(basic_conf)
        <class 'insights.parsers.insights_client_conf.BasicAuthInsightsClient'>
        >>> basic_conf['username_set']
        True
    """
    pass



"""
GCPNetworkInterfaces
====================

This parser reads the output of a command
``curl -H "Metadata-Flavor: Google" "http://metadata/computeMetadata/v1/instance/network-interfaces/?recursive=true"``,
which is used to fetch external IP address information.

For more details, See: https://cloud.google.com/compute/docs/metadata/overview

"""
import json

from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.gcp_network_interfaces)
class GCPNetworkInterfaces(CommandParser):
    """
    Class for parsing the GCP License Codes returned by command
    ``curl -H "Metadata-Flavor: Google" "http://metadata/computeMetadata/v1/instance/network-interfaces/?recursive=true"``,

    Typical Output of this command is::

        [
           {
              "accessConfigs":[
                 {
                    "externalIp":"34.67.41.222",
                    "type":"ONE_TO_ONE_NAT"
                 }
              ],
              "dnsServers":[
                 "169.254.169.254"
              ],
              "forwardedIps":[],
              "gateway":"10.128.0.1",
              "ip":"10.128.0.3",
              "ipAliases":[],
              "mac":"42:01:0a:80:00:03",
              "mtu":1460,
              "network":"projects/564421043971/networks/default",
              "subnetmask":"255.255.240.0",
              "targetInstanceIps":[]
           }
        ]

    Raises:
        SkipComponent: When content is empty or no parse-able content.
        ParseException: When the json is unable to be parsed

    Attributes:
        public_ips (list): A list containing public IPs in no particular order

    Examples:
        >>> gcp_network_interfaces.public_ips
        ["34.67.41.222"]
    """

    def parse_content(self, content):
        if not content or 'curl: ' in content[0]:
            raise SkipComponent()
        try:
            self.raw = json.loads(' '.join(content))
        except:
            raise ParseException("Unable to parse JSON")

        self.public_ips = []
        for nic in self.raw:
            if "accessConfigs" in nic:
                for ac in nic["accessConfigs"]:
                    if "externalIp" in ac:
                        self.public_ips.append(ac["externalIp"])

    def __repr__(self):
        return "public_ips: {i}, raw: {r}".format(i=self.public_ips, r=self.raw)



"""
NginxConfTree - Combiner for nginx configuration
================================================
"""
from insights.core import ConfigCombiner, ContainerConfigCombiner
from insights.core.plugins import combiner
from insights.parsers.nginx_conf import NginxConfPEG, ContainerNginxConfPEG
from insights.parsr.query import eq
from os.path import dirname


@combiner(NginxConfPEG)
class NginxConfTree(ConfigCombiner):
    """
    This module models nginx configuration as a tree. It correctly handles include
    directives by splicing individual document trees into their parents until one
    document tree is left.

    A DSL is provided to query the tree through a select function or brackets [].
    The brackets allow a more conventional lookup feel but aren't quite as powerful
    as using select directly.

    See the :py:class:`insights.core.ConfigComponent` class for example usage.
    """
    def __init__(self, confs):
        super(NginxConfTree, self).__init__(confs, "nginx.conf", eq("include"))

    @property
    def conf_path(self):
        return dirname(self.main.file_path)


@combiner(ContainerNginxConfPEG)
class ContainerNginxConfTree(list):
    """
    This module models the nginx configuration of the same running containers
    as a tree and wrap the `tree` of containers into a list.
    Within the tree, It correctly handles include directives by splicing
    individual document trees into their parents until one document tree is
    left.

    See the :py:class:`insights.core.ConfigComponent` class for example usage.
    """
    def __init__(self, ctn_confs):
        containers = {}
        for ctn in ctn_confs:
            if ctn.container_id not in containers:
                containers[ctn.container_id] = [ctn]
            else:
                containers[ctn.container_id].append(ctn)

        for ctn_id, confs in containers.items():
            self.append(ContainerConfigCombiner(
                    confs, "nginx.conf", eq("include"),
                    confs[0].engine, confs[0].image, ctn_id))



"""
Libssh configuration files
==========================

This module provides the ssh options in libssh configuration file

Parsers provided by this module are:

LibsshClientConfig - file ``/etc/libssh/libssh_client.config``
--------------------------------------------------------------

LibsshServerConfig - file ``/etc/libssh/libssh_server.config``
--------------------------------------------------------------
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


class LibsshConfig(CommandParser, dict):
    """
    Base class for libssh configuration file.

    There are many options in the libssh configuration file and
    all of them are stored in one of the following formations:
    a)   Key  [space]  Value
    b)   Key  =  Value

    There are the keywords used in the configuration file:
        include
        hostkey
        listenaddress
        port
        loglevel
        ciphers
        macs
        kexalgorithms
        match
        pubkeyacceptedkeytypes
        hostkeyalgorithms
        all
        user
        group
        host
        localaddress
        localport
        rdomain
        address

    Sample output::

        # Parse system-wide crypto configuration file
        Include /etc/crypto-policies/back-ends/libssh.config
        # Parse OpenSSH configuration file for consistency
        Include /etc/ssh/sshd_config

    .. note::

        If there are two or more lines that have the same key, then we
        store the values in a list.

    Examples:
        >>> 'Include' in config
        True
        >>> config['Include']
        ['/etc/crypto-policies/back-ends/libssh.config', '/etc/ssh/sshd_config']

    Raises:
        SkipComponent: When input content is empty or there is a syntax error.
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent('No content.')

        for line in get_active_lines(content):
            delimiter = None
            if ' ' in line:
                delimiter = ' '

            if '=' in line:
                delimiter = '='

            try:
                k, v = [i.strip() for i in line.split(delimiter)]
                if k not in self:
                    self[k] = v
                else:
                    _v = self[k]
                    _v = [_v] if not isinstance(_v, list) else _v
                    if v not in _v:
                        _v.append(v)
                        self[k] = _v
            except ValueError:
                raise SkipComponent('Syntax error')


@parser(Specs.libssh_client_config)
class LibsshClientConfig(LibsshConfig):
    """
    Parser for accessing the ``/etc/libssh/libssh_client.config`` file.

    .. note::

        Please refer to the super-class :py:class:`insights.parsers.libssh_config:LibsshConfig`
        for more usage information.
    """
    pass


@parser(Specs.libssh_server_config)
class LibsshServerConfig(LibsshConfig):
    """
    Parser for accessing the ``/etc/libssh/libssh_server.config`` file

    .. note::

        Please refer to the super-class :py:class:`insights.parsers.libssh_config:LibsshConfig`
        for more usage information.
    """
    pass



from insights.core.context import OSP
from insights.parsers.rabbitmq import RabbitMQUsers
from insights.tests import context_wrap

osp_controller = OSP()
osp_controller.role = "Controller"

RABBITMQ_LIST_USERS = """
Listing users ...
guest   [administrator]
test    [administrator]
...done.
"""

RABBITMQ_LIST_EDGES = '''
Listing users ...
probe   []
brain   []
none
user1   [monitoring,user]
guest   [made up data]
...done.
'''


def test_rabbitmq_list_users():
    context = context_wrap(RABBITMQ_LIST_USERS, hostname="controller_1", osp=osp_controller)
    result = RabbitMQUsers(context)
    expect = {"guest": "administrator", "test": "administrator"}
    assert result.data == expect


def test_rabbitmq_list_users_stub():
    context = context_wrap(RABBITMQ_LIST_EDGES, hostname="controller_1", osp=osp_controller)
    result = RabbitMQUsers(context)
    assert result.data['probe'] == ''
    assert 'none' not in result.data
    assert result.data['user1'] == 'monitoring,user'
    assert result.data['guest'] == 'made up data'



"""
NetworkManagerDhclient - file ``/etc/NetworkManager/dispatcher.d/*-dhclient``
=============================================================================
"""
import re

from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


VULNERABLE_BLOCK_RHEL_6 = re.compile(re.escape("""
eval "$(
declare | LC_ALL=C grep '^DHCP4_[A-Z_]*=' | while read opt; do
    optname=${opt%%=*}
    optname=${optname,,}
    optname=new_${optname#dhcp4_}
    optvalue=${opt#*=}
    echo "$optname=$optvalue"
done
)"
""".strip()))

VULNERABLE_BLOCK_RHEL_7 = re.compile(re.escape("""
eval "$(
declare | LC_ALL=C grep '^DHCP4_[A-Z_]*=' | while read opt; do
    optname=${opt%%=*}
    optname=${optname,,}
    optname=new_${optname#dhcp4_}
    optvalue=${opt#*=}
    echo "export $optname=$optvalue"
done
)"
""".strip()))


@parser(Specs.networkmanager_dispatcher_d)
class NetworkManagerDhclient(Parser):
    """
    Class for parsing ``/etc/NetworkManager/dispatcher.d/*-dhclient`` file.

    Attributes:
        has_vulnerable_block (bool): True, if the vulnerable block is present, False otherwise.

    Raises:
        SkipComponent: When content is empty or cannot be parsed.

    Sample output of this command is::

        #!/bin/bash
        # run dhclient.d scripts in an emulated environment

        PATH=/bin:/usr/bin:/sbin
        SAVEDIR=/var/lib/dhclient
        ETCDIR=/etc/dhcp
        interface=$1

        eval "$(
        declare | LC_ALL=C grep '^DHCP4_[A-Z_]*=' | while read opt; do
            optname=${opt%%=*}
            optname=${optname,,}
            optname=new_${optname#dhcp4_}
            optvalue=${opt#*=}
            echo "export $optname=$optvalue"
        done
        )"

        [ -f /etc/sysconfig/network ] && . /etc/sysconfig/network

        [ -f /etc/sysconfig/network-scripts/ifcfg-$interface ] && \
            . /etc/sysconfig/network-scripts/ifcfg-$interface

        if [ -d $ETCDIR/dhclient.d ]; then
            for f in $ETCDIR/dhclient.d/*.sh; do
                if [ -x $f ]; then
                    subsystem="${f%.sh}"
                    subsystem="${subsystem##*/}"
                    . ${f}
                    if [ "$2" = "up" ]; then
                        "${subsystem}_config"
                    elif [ "$2" = "dhcp4-change" ]; then
                        if [ "$subsystem" = "chrony" -o "$subsystem" = "ntp" ]; then
                            "${subsystem}_config"
                        fi
                    elif [ "$2" = "down" ]; then
                        "${subsystem}_restore"
                    fi
                fi
            done
        fi

    Examples:
        >>> type(dhclient)
        <class 'insights.parsers.networkmanager_dhclient.NetworkManagerDhclient'>
        >>> dhclient.has_vulnerable_block
        True
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("No content.")

        result = "\n".join(content)
        match_rhel_6 = VULNERABLE_BLOCK_RHEL_6.search(result)
        match_rhel_7 = VULNERABLE_BLOCK_RHEL_7.search(result)
        self.has_vulnerable_block = bool(match_rhel_7) or bool(match_rhel_6)



"""
SCSIFWver - file ``/sys/class/scsi_host/host[0-9]*/fwrev``
==========================================================

This parser parses the content from fwver file from individual
SCSI hosts. This parser will return data in dictionary format.

Sample Content from ``/sys/class/scsi_host/host0/fwrev``::

    2.02X12 (U3H2.02X12), sli-3


Examples:
    >>> type(scsi_obj)
    <class 'insights.parsers.scsi_fwver.SCSIFWver'>
    >>> scsi_obj.data
    {'host0': ['2.02X12 (U3H2.02X12)', 'sli-3']}
    >>> scsi_obj.scsi_host
    'host0'
"""

from insights import Parser, parser, get_active_lines, LegacyItemAccess
from insights.specs import Specs


@parser(Specs.scsi_fwver)
class SCSIFWver(LegacyItemAccess, Parser):
    """
    Parse `/sys/class/scsi_host/host[0-9]*/fwrev` file, return a dict
    contain `fwver` scsi host file info. "scsi_host" key is scsi host file
    parse from scsi host file name.

    Properties:
        scsi_host (str): scsi host file name derived from file path.
    """

    def __init__(self, context):
        self.data = {}
        self.scsi_host = context.path.rsplit("/")[-2]
        super(SCSIFWver, self).__init__(context)

    def parse_content(self, content):
        for line in get_active_lines(content):
            self.data[self.scsi_host] = [mode.strip() for mode in line.split(',')]

    @property
    def host_mode(self):
        """
        (list): It will return the scsi host modes when set else `None`.
        """
        return self.data[self.scsi_host]



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import systemctl_get_default
from insights.parsers.systemctl_get_default import SystemctlGetDefault
from insights.tests import context_wrap


SYSTEMCTL_GET_DEFAULT_CONTENT = """
graphical.target
""".strip()

SYSTEMCTL_GET_DEFAULT_EMPTY = """

""".strip()


def test_systemctl_get_default():
    result = SystemctlGetDefault(context_wrap(SYSTEMCTL_GET_DEFAULT_CONTENT))
    assert result.default_target == 'graphical.target'


def test_systemctl_get_default_empty():
    with pytest.raises(SkipComponent):
        SystemctlGetDefault(context_wrap(SYSTEMCTL_GET_DEFAULT_EMPTY))


def test_systemctl_get_default_doc():
    env = {
        'systemctl_get_default': SystemctlGetDefault(context_wrap(SYSTEMCTL_GET_DEFAULT_CONTENT))
    }
    failed, total = doctest.testmod(systemctl_get_default, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import slabinfo
from insights.parsers.slabinfo import SlabInfo
from insights.tests import context_wrap


PROC_SLABINFO = """
slabinfo - version: 2.1
# name            <active_objs> <num_objs> <objsize> <objperslab> <pagesperslab> : tunables <limit> <batchcount> <sharedfactor> : slabdata <active_slabs> <num_slabs> <sharedavail>
sw_flow                0      0   1256   13    4 : tunables    0    0    0 : slabdata      0      0      0
nf_conntrack_ffffffffaf313a40     12     12    320   12    1 : tunables    10   20    30 : slabdata      40      50      60
xfs_dqtrx              0      0    528   15    2 : tunables    0    0    0 : slabdata      0      0      0
xfs_dquot              0      0    488    8    1 : tunables    0    0    0 : slabdata      0      0      0
xfs_ili             2264   2736    168   24    1 : tunables    0    0    0 : slabdata    114    114      0
xfs_inode           4857   5120    960    8    2 : tunables    0    0    0 : slabdata    640    640      0
xfs_efd_item          76     76    416   19    2 : tunables    0    0    0 : slabdata      4      4      0
xfs_btree_cur         18     18    216   18    1 : tunables    0    0    0 : slabdata      1      1      0
xfs_log_ticket        22     22    184   22    1 : tunables    0    0    0 : slabdata      1      1      0
bio-3                 60     60    320   12    1 : tunables    0    0    0 : slabdata      5      5      0
kcopyd_job             0      0   3312    9    8 : tunables    0    0    0 : slabdata      0      0      0
dm_uevent              0      0   2608   12    8 : tunables    0    0    0 : slabdata      0      0      0
dm_rq_target_io        0      0    136   30    1 : tunables    0    0    0 : slabdata      0      0      0
ip6_dst_cache         72     72    448    9    1 : tunables    0    0    0 : slabdata      8      8      0
RAWv6                 13     13   1216   13    4 : tunables    0    0    0 : slabdata      1      1      0
UDPLITEv6              0      0   1216   13    4 : tunables    0    0    0 : slabdata      0      0      0
UDPv6                 13     13   1216   13    4 : tunables    0    0    0 : slabdata      1      1      0
tw_sock_TCPv6          0      0    256   16    1 : tunables    0    0    0 : slabdata      0      0      0
TCPv6                 15     15   2112   15    8 : tunables    0    0    0 : slabdata      1      1      0
cfq_queue              0      0    232   17    1 : tunables    0    0    0 : slabdata      0      0      0
bsg_cmd                0      0    312   13    1 : tunables    0    0    0 : slabdata      0      0      0
mqueue_inode_cache      9      9    896    9    2 : tunables    0    0    0 : slabdata      1      1      0
hugetlbfs_inode_cache     13     13    608   13    2 : tunables    0    0    0 : slabdata      1      1      0
configfs_dir_cache      0      0     88   46    1 : tunables    0    0    0 : slabdata      0      0      0
dquot                  0      0    256   16    1 : tunables    0    0    0 : slabdata      0      0      0
kioctx                 0      0    576   14    2 : tunables    0    0    0 : slabdata      0      0      0
userfaultfd_ctx_cache      0      0    192   21    1 : tunables    0    0    0 : slabdata      0      0      0
dio                   12     12    640   12    2 : tunables    0    0    0 : slabdata      1      1      0
pid_namespace          0      0   2200   14    8 : tunables    0    0    0 : slabdata      0      0      0
posix_timers_cache     64     64    248   16    1 : tunables    0    0    0 : slabdata      4      4      0
UDP-Lite               0      0   1088   15    4 : tunables    0    0    0 : slabdata      0      0      0
flow_cache             0      0    144   28    1 : tunables    0    0    0 : slabdata      0      0      0
UDP                   15     15   1088   15    4 : tunables    0    0    0 : slabdata      1      1      0
tw_sock_TCP            0      0    256   16    1 : tunables    0    0    0 : slabdata      0      0      0
TCP                    8      8   1984    8    4 : tunables    0    0    0 : slabdata      1      1      0
dax_cache             10     10    768   10    2 : tunables    0    0    0 : slabdata      1      1      0
blkdev_integrity       0      0    112   36    1 : tunables    0    0    0 : slabdata      0      0      0
blkdev_queue          13     13   2456   13    8 : tunables    0    0    0 : slabdata      1      1      0
blkdev_ioc            78     78    104   39    1 : tunables    0    0    0 : slabdata      2      2      0
user_namespace         8      8    480    8    1 : tunables    0    0    0 : slabdata      1      1      0
dmaengine-unmap-128     15     15   1088   15    4 : tunables    0    0    0 : slabdata      1      1      0
sock_inode_cache     216    216    640   12    2 : tunables    0    0    0 : slabdata     18     18      0
fsnotify_mark_connector 136680 136680     24  170    1 : tunables    0    0    0 : slabdata    804    804      0
net_namespace          6      6   5248    6    8 : tunables    0    0    0 : slabdata      1      1      0
shmem_inode_cache    840    840    680   12    2 : tunables    0    0    0 : slabdata     70     70      0
Acpi-State           153    153     80   51    1 : tunables    0    0    0 : slabdata      3      3      0
task_delay_info      180    180    112   36    1 : tunables    0    0    0 : slabdata      5      5      0
taskstats             12     12    328   12    1 : tunables    0    0    0 : slabdata      1      1      0
proc_inode_cache    1199   1308    656   12    2 : tunables    0    0    0 : slabdata    109    109      0
sigqueue              25     25    160   25    1 : tunables    0    0    0 : slabdata      1      1      0
bdev_cache            19     19    832   19    4 : tunables    0    0    0 : slabdata      1      1      0
kernfs_node_cache  15028  15028    120   34    1 : tunables    0    0    0 : slabdata    442    442      0
mnt_cache            907    990    384   10    1 : tunables    0    0    0 : slabdata     99     99      0
inode_cache        14183  14183    592   13    2 : tunables    0    0    0 : slabdata   1091   1091      0
dentry             25349  25662    192   21    1 : tunables    0    0    0 : slabdata   1222   1222      0
iint_cache             0      0    128   32    1 : tunables    0    0    0 : slabdata      0      0      0
avc_xperms_node      292    292     56   73    1 : tunables    0    0    0 : slabdata      4      4      0
avc_node            2408   2408     72   56    1 : tunables    0    0    0 : slabdata     43     43      0
selinux_inode_security  22032  22032     40  102    1 : tunables    0    0    0 : slabdata    216    216      0
buffer_head         4017   4407    104   39    1 : tunables    0    0    0 : slabdata    113    113      0
vm_area_struct      3880   4194    216   18    1 : tunables    0    0    0 : slabdata    233    233      0
mm_struct             50     50   1600   10    4 : tunables    0    0    0 : slabdata      5      5      0
fs_cache              64     64     64   64    1 : tunables    0    0    0 : slabdata      1      1      0
files_cache           48     48    640   12    2 : tunables    0    0    0 : slabdata      4      4      0
signal_cache         127    154   1152   14    4 : tunables    0    0    0 : slabdata     11     11      0
sighand_cache        107    135   2112   15    8 : tunables    0    0    0 : slabdata      9      9      0
task_xstate          171    171    832   19    4 : tunables    0    0    0 : slabdata      9      9      0
task_struct          122    147   4208    7    8 : tunables    0    0    0 : slabdata     21     21      0
cred_jar             609    609    192   21    1 : tunables    0    0    0 : slabdata     29     29      0
anon_vma            1836   1836     80   51    1 : tunables    0    0    0 : slabdata     36     36      0
pid                  192    192    128   32    1 : tunables    0    0    0 : slabdata      6      6      0
shared_policy_node   2635   2635     48   85    1 : tunables    0    0    0 : slabdata     31     31      0
numa_policy           15     15    264   15    1 : tunables    0    0    0 : slabdata      1      1      0
radix_tree_node     2878   3052    584   14    2 : tunables    0    0    0 : slabdata    218    218      0
idr_layer_cache      180    180   2112   15    8 : tunables    0    0    0 : slabdata     12     12      0
dma-kmalloc-8192       0      0   8192    4    8 : tunables    0    0    0 : slabdata      0      0      0
dma-kmalloc-4096       0      0   4096    8    8 : tunables    0    0    0 : slabdata      0      0      0
dma-kmalloc-2048       0      0   2048    8    4 : tunables    0    0    0 : slabdata      0      0      0
dma-kmalloc-1024       0      0   1024    8    2 : tunables    0    0    0 : slabdata      0      0      0
dma-kmalloc-512        8      8    512    8    1 : tunables    0    0    0 : slabdata      1      1      0
dma-kmalloc-256        0      0    256   16    1 : tunables    0    0    0 : slabdata      0      0      0
dma-kmalloc-128        0      0    128   32    1 : tunables    0    0    0 : slabdata      0      0      0
dma-kmalloc-64         0      0     64   64    1 : tunables    0    0    0 : slabdata      0      0      0
dma-kmalloc-32         0      0     32  128    1 : tunables    0    0    0 : slabdata      0      0      0
dma-kmalloc-16         0      0     16  256    1 : tunables    0    0    0 : slabdata      0      0      0
dma-kmalloc-8          0      0      8  512    1 : tunables    0    0    0 : slabdata      0      0      0
dma-kmalloc-192        0      0    192   21    1 : tunables    0    0    0 : slabdata      0      0      0
dma-kmalloc-96         0      0     96   42    1 : tunables    0    0    0 : slabdata      0      0      0
kmalloc-8192          40     40   8192    4    8 : tunables    0    0    0 : slabdata     10     10      0
kmalloc-4096         255    272   4096    8    8 : tunables    0    0    0 : slabdata     34     34      0
kmalloc-2048         280    296   2048    8    4 : tunables    0    0    0 : slabdata     37     37      0
kmalloc-1024        1135   1152   1024    8    2 : tunables    0    0    0 : slabdata    144    144      0
kmalloc-512          649    760    512    8    1 : tunables    0    0    0 : slabdata     95     95      0
kmalloc-256         2735   2992    256   16    1 : tunables    0    0    0 : slabdata    187    187      0
kmalloc-192         1407   1407    192   21    1 : tunables    0    0    0 : slabdata     67     67      0
kmalloc-128         1664   1664    128   32    1 : tunables    0    0    0 : slabdata     52     52      0
kmalloc-96          1470   1470     96   42    1 : tunables    0    0    0 : slabdata     35     35      0
kmalloc-64         40126  40640     64   64    1 : tunables    0    0    0 : slabdata    635    635      0
kmalloc-32        134702 135552     32  128    1 : tunables    0    0    0 : slabdata   1059   1059      0
kmalloc-16         65792  65792     16  256    1 : tunables    0    0    0 : slabdata    257    257      0
kmalloc-8          87552  87552      8  512    1 : tunables    0    0    0 : slabdata    171    171      0
kmem_cache_node      128    128     64   64    1 : tunables    0    0    0 : slabdata      2      2      0
kmem_cache           112    111    256   16    1 : tunables    10   20   30 : slabdata    17     18     19
""".strip()

SLABINFO_DOC = """
slabinfo - version: 2.1
# name            <active_objs> <num_objs> <objsize> <objperslab> <pagesperslab> : tunables <limit> <batchcount> <sharedfactor> : slabdata <active_slabs> <num_slabs> <sharedavail>
sw_flow                0      0   1256   13    4 : tunables    0    0    0 : slabdata      0      0      0
nf_conntrack_ffffffffaf313a40     12     12    320   12    1 : tunables    0    0    0 : slabdata      1      1      0
xfs_dqtrx              0      0    528   15    2 : tunables    0    0    0 : slabdata      0      0      0
xfs_dquot              0      0    488    8    1 : tunables    0    0    0 : slabdata      0      0      0
xfs_ili             2264   2736    168   24    1 : tunables    0    0    0 : slabdata    114    114      0
xfs_inode           4845   5120    960    8    2 : tunables    0    0    0 : slabdata    640    640      0
xfs_efd_item          76     76    416   19    2 : tunables    0    0    0 : slabdata      4      4      0
xfs_btree_cur         18     18    216   18    1 : tunables    0    0    0 : slabdata      1      1      0
xfs_log_ticket        22     22    184   22    1 : tunables    0    0    0 : slabdata      1      1      0
bio-3                 60     60    320   12    1 : tunables    0    0    0 : slabdata      5      5      0
kcopyd_job             0      0   3312    9    8 : tunables    0    0    0 : slabdata      0      0      0
dm_uevent              0      0   2608   12    8 : tunables    0    0    0 : slabdata      0      0      0
dm_rq_target_io        0      0    136   30    1 : tunables    0    0    0 : slabdata      0      0      0
ip6_dst_cache         72     72    448    9    1 : tunables    0    0    0 : slabdata      8      8      0
RAWv6                 13     13   1216   13    4 : tunables    0    0    0 : slabdata      1      1      0
UDPLITEv6              0      0   1216   13    4 : tunables    0    0    0 : slabdata      0      0      0
UDPv6                 13     13   1216   13    4 : tunables    0    0    0 : slabdata      1      1      0
tw_sock_TCPv6          0      0    256   16    1 : tunables    0    0    0 : slabdata      0      0      0
TCPv6                 15     15   2112   15    8 : tunables    0    0    0 : slabdata      1      1      0
cfq_queue              0      0    232   17    1 : tunables    0    0    0 : slabdata      0      0      0
bsg_cmd                0      0    312   13    1 : tunables    10   20   30 : slabdata     40     50     60
""".strip()

SLABINFO_DETAILS = """
""".strip()

SLABINFO_DETAILS_2 = """
# name            <active_objs> <num_objs> <objsize> <objperslab> <pagesperslab> : tunables <limit> <batchcount> <sharedfactor> : slabdata <active_slabs> <num_slabs> <sharedavail>
cfq_queue              0      0    232   17    1 : tunables    0    0    0 : slabdata      0      0      0
bsg_cmd                0      0    312   13    1 : tunables    10   20   30 : slabdata     40     50     60
""".strip()

SLABINFO_DETAILS_3 = """
slabinfo - version: 2.1
cfq_queue              0      0    232   17    1 : tunables    0    0    0 : slabdata      0      0      0
bsg_cmd                0      0    312   13    1 : tunables    10   20   30 : slabdata     40     50     60
""".strip()

SLABINFO_DETAILS_LEN_MISS_MATCH = """
slabinfo - version: 2.1
# name            <active_objs> <num_objs> <objsize> <objperslab> <pagesperslab> : tunables <limit> <batchcount> <sharedfactor> : slabdata <active_slabs> <num_slabs> <sharedavail>
sw_flow                0      0   1256   13    4 : tunables    0    0    0 : slabdata      0      0      0      100     1220
nf_conntrack_ffffffffaf313a40     12     12    320   12    1 : tunables    0    0    0 : slabdata      1      1      10     100
""".strip()


def test_check_slabinfo():
    result = SlabInfo(context_wrap(PROC_SLABINFO))
    assert len(result.data.keys()) == 103
    assert result.slab_object('kcopyd_job', 'objsize') == 3312
    assert result.slab_object('kmalloc-16', 'objsize') == 16
    assert result.slab_object('kmalloc-16', 'name') is None
    assert result.slab_version == '2.1'
    result.slab_details('kmem_cache') == {'name': 'kmem_cache', 'active_objs': '112', 'num_objs': '111', 'objsize': '256', 'objperslab': '16', 'pagesperslab': '1', 'limit': '10', 'batchcount': '20', 'sharedfactor': '30', 'active_slabs': '17', 'num_slabs': '18', 'sharedavail': '19'}


def test_slabinfo_doc_examples():
    env = {
        'pslabinfo': SlabInfo(context_wrap(SLABINFO_DOC))
    }
    failed, total = doctest.testmod(slabinfo, globs=env)
    assert failed == 0


def test_slabinfo_exception():
    with pytest.raises(SkipComponent) as exc:
        pslabinfo = SlabInfo(context_wrap(SLABINFO_DETAILS))
        assert pslabinfo is None
    assert 'No Contents' in str(exc)

    with pytest.raises(SkipComponent) as exc:
        pslabinfo_1 = SlabInfo(context_wrap(SLABINFO_DETAILS_2))
        assert pslabinfo_1 is None
    assert 'Invalid Contents' in str(exc)

    with pytest.raises(SkipComponent) as exc:
        pslabinfo_2 = SlabInfo(context_wrap(SLABINFO_DETAILS_3))
        assert pslabinfo_2 is None
    assert 'Invalid Contents' in str(exc)

    with pytest.raises(SkipComponent) as exc_1:
        pslabinfo_3 = SlabInfo(context_wrap(SLABINFO_DETAILS_LEN_MISS_MATCH))
        assert pslabinfo_3 is None
    assert 'Invalid Contents' in str(exc_1)



"""
Custom datasources to get the db2 users and database names.
"""

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.combiners.ps import Ps
from insights.core.spec_factory import foreach_execute
from insights.specs import Specs


@datasource(Ps, HostContext)
def db2_users(broker):
    """
    This datasource get the name of the users running db2 databases

    Returns:
        list: the names of the users running db2 databases

    Raises:
        SkipComponent: there is no db2 users running db2 databases
    """
    ps = broker[Ps]
    ps_list = ps.search(COMMAND_NAME__contains="db2sysc")
    if ps_list:
        users_result = set(filter(None, [item.get("USER") for item in ps_list]))
        users_result.discard('root')
        if users_result:
            return sorted(users_result)
        raise SkipComponent("No db2 user is available")
    raise SkipComponent("No db2 database is running")


class LocalSpecs(Specs):
    db2_databases = foreach_execute(db2_users, "/usr/sbin/runuser -l  %s  -c 'db2 list database directory'")


@datasource(LocalSpecs.db2_databases, HostContext)
def db2_databases_info(broker):
    """
    This datasource get the name of the users running db2 databases and the database names

    Returns:
        list: the set of user names and database names

    Raises:
        SkipComponent: there is no db2 databases
    """
    result = set()
    for item in broker[LocalSpecs.db2_databases]:
        user = item.cmd.split()[2].strip()
        for line in item.content:
            if "Database name" in line:
                name = line.split("=")[-1].strip()
                result.add((user, name))
    if result:
        return sorted(result)
    raise SkipComponent("No db2 database is available")



import doctest
from insights.parsers import cpu_online
from insights.tests import context_wrap


CPU_ONLINE = """
0,2-4,7
""".strip()


def test_container_cpu_online():
    container_cpu_online_info = cpu_online.ContainerCpuOnline(
        context_wrap(
            CPU_ONLINE,
            container_id='2869b4e2541c',
            image='registry.access.redhat.com/ubi8/nginx-120',
            engine='podman',
            path='insights_containers/2869b4e2541c/sys/fs/cgroup/cpuset/cpuset.cpus'
        )
    )
    assert container_cpu_online_info.cpu_online_set == ["0", "2", "3", "4", "7"]
    assert container_cpu_online_info.cpu_online_number == 5


def test_doc():
    env = {
        "container_cpu_online_info": cpu_online.ContainerCpuOnline(
            context_wrap(
                CPU_ONLINE,
                container_id='2869b4e2541c',
                image='registry.access.redhat.com/ubi8/nginx-120',
                engine='podman'
            )
        )
    }
    failed_count, total = doctest.testmod(cpu_online, globs=env)
    assert failed_count == 0



"""
Kerberos KDC Logs - file ``/var/log/krb5kdc.log``
=================================================

"""

from .. import LogFileOutput, parser

import re
from insights.specs import Specs


@parser(Specs.kerberos_kdc_log)
class KerberosKDCLog(LogFileOutput):
    '''
    Read the ``/var/log/krb5kdc.log`` file.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput` for
        more usage information.

    Find logs by keyword and parse them into a dictionary with the keys:

        * `timestamp`
        * `system`
        * `service`
        * `pid`
        * `level`
        * `message`
        * `raw_message` - the full line as originally given.

    If the log line is not in the standard format, only the `raw_message` field
    will be stored in the dictionary.

    Sample log file::

        Apr 01 03:36:11 ldap.example.com krb5kdc[24569](info): TGS_REQ (4 etypes {18 17 16 23}) 10.250.3.150: ISSUE: authtime 1427873771, etypes {rep=18 tkt=18 ses=18}, sasher@EXAMPLE.COM for HTTP/sepdt138.example.com@EXAMPLE.COM
        Apr 01 03:36:11 ldap.example.com krb5kdc[24569](info): AS_REQ (4 etypes {18 17 16 23}) 10.250.17.96: NEEDED_PREAUTH: niz@EXAMPLE.COM for krbtgt/EXAMPLE.COM@EXAMPLE.COM, Additional pre-authentication required
        Apr 01 03:36:11 ldap.example.com krb5kdc[24549](info): AS_REQ (4 etypes {18 17 16 23}) 10.250.17.96: NEEDED_PREAUTH: niz@EXAMPLE.COM for krbtgt/EXAMPLE.COM@EXAMPLE.COM, Additional pre-authentication required
        Apr 01 03:36:11 ldap.example.com krb5kdc[24546](info): AS_REQ (4 etypes {18 17 16 23}) 10.250.17.96: NEEDED_PREAUTH: niz@EXAMPLE.COM for krbtgt/EXAMPLE.COM@EXAMPLE.COM, Additional pre-authentication required
        Apr 01 03:36:33 ldap.example.com krb5kdc[24556](info): preauth (encrypted_timestamp) verify failure: Decrypt integrity check failed
        Apr 01 03:36:36 ldap.example.com krb5kdc[24568](info): preauth (encrypted_timestamp) verify failure: No matching key in entry
        Apr 01 03:38:34 ldap.example.com krb5kdc[24551](info): preauth (encrypted_timestamp) verify failure: No matching key in entry
        Apr 01 03:39:43 ldap.example.com krb5kdc[24549](info): preauth (encrypted_timestamp) verify failure: No matching key in entry

    Examples:
        >>> log = shared[KerberosKDCLog]
        >>> # log.get is a generator, so get list to test length
        >>> len(list(log.get('Decrypt integrity check failed')))
        1
        >>> from datetime import datetime
        >>> len(log.get_after(datetime(2017, 4, 1, 3, 36, 30)))  # Apr 01 03:36:30
        4


    .. note::
        Because the Kerberos KDC log timestamps by default have no year,
        the year of the logs will be inferred from the year in your timestamp.
        This will also work around December/January crossovers.
    '''
    time_format = '%b %d %H:%M:%S'
    _LINE_RE = re.compile(
        r'(?P<timestamp>\w{3} \d\d \d\d:\d\d:\d\d) ' +
        r'(?P<system>\w\S+) ' +
        r'(?P<service>\w+)\[(?P<pid>\d+)\]\((?P<level>\w+)\): ' +
        r'(?P<message>.*)'
    )

    def _parse_line(self, line):
        data = {'raw_message': line}
        match = self._LINE_RE.search(line)
        if match:
            data.update(match.groupdict())
        return data



"""
AuthSelectCurrent - command ``authselect current``
==================================================
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.authselect_current)
class AuthSelectCurrent(CommandParser):
    """
    Class to parse the output of command "authselect current".

    Sample output of command ``authselect current``::

        Profile ID: sssd
        Enabled features:
        - with-sudo
        - with-mkhomedir
        - with-smartcard

    Attributes:
        profile_id (str): The enabled profile ID
        enabled_features (list): List of enabled features

    Examples:
        >>> asc.profile_id
        'sssd'
        >>> len(asc.enabled_features)
        3
        >>> 'with-sudo' in asc.enabled_features
        True
    """
    def parse_content(self, content):
        feature_flag = False
        self.profile_id = None
        self.enabled_features = []
        for line in content:
            if 'No existing configuration detected.' in line:
                raise SkipComponent
            line_sp = line.split()
            if line.startswith('Profile ID:'):
                self.profile_id = line_sp[-1]
            elif line.endswith('Enabled features:'):
                # Enabled features:
                # - ...
                feature_flag = True
            elif line.startswith('Enabled features:') and len(line_sp) > 2:
                # Enabled features: None
                feature_flag = True
                if line_sp[-1] != 'None':
                    self.enabled_features.append(line_sp[-1])
            elif line.startswith('-') and feature_flag:
                self.enabled_features.append(line_sp[-1])
        if self.profile_id is None:
            raise SkipComponent
